@@ -6,6 +6,7 @@
 
 #include "nsNSSCallbacks.h"
 
+#include "NSSSocketControl.h"
 #include "PSMRunnable.h"
 #include "ScopedNSSTypes.h"
 #include "SharedCertVerifier.h"
@@ -13,17 +14,22 @@
 #include "mozilla/ArrayUtils.h"
 #include "mozilla/Assertions.h"
 #include "mozilla/Casting.h"
+#include "mozilla/Logging.h"
 #include "mozilla/RefPtr.h"
+#include "mozilla/ScopeExit.h"
 #include "mozilla/Span.h"
+#include "mozilla/SpinEventLoopUntil.h"
 #include "mozilla/Telemetry.h"
 #include "mozilla/Unused.h"
+#include "mozilla/intl/Localization.h"
 #include "nsContentUtils.h"
+#include "nsIChannel.h"
+#include "nsIHttpChannel.h"
 #include "nsIHttpChannelInternal.h"
 #include "nsIPrompt.h"
 #include "nsIProtocolProxyService.h"
 #include "nsISupportsPriority.h"
 #include "nsIStreamLoader.h"
-#include "nsITokenDialogs.h"
 #include "nsIUploadChannel.h"
 #include "nsIWebProgressListener.h"
 #include "nsNSSCertHelper.h"
@@ -32,7 +38,6 @@
 #include "nsNSSHelper.h"
 #include "nsNSSIOLayer.h"
 #include "nsNetUtil.h"
-#include "nsProtectedAuthThread.h"
 #include "nsProxyRelease.h"
 #include "nsStringStream.h"
 #include "mozpkix/pkixtypes.h"
@@ -43,7 +48,6 @@
 #include "TrustOverrideUtils.h"
 #include "TrustOverride-SymantecData.inc"
 #include "TrustOverride-AppleGoogleDigiCertData.inc"
-#include "TrustOverride-TestImminentDistrustData.inc"
 
 using namespace mozilla;
 using namespace mozilla::pkix;
@@ -102,7 +106,7 @@ class OCSPRequest final : public nsIStreamLoaderObserver, public nsIRunnable {
   // cancelled. This is how we know the closure in OnTimeout is valid. If the
   // timer fires before OnStreamComplete runs, it should be safe to not cancel
   // the request because necko has a strong reference to it.
-  Monitor mMonitor;
+  Monitor mMonitor MOZ_UNANNOTATED;
   bool mNotifiedDone;
   nsCOMPtr<nsIStreamLoader> mLoader;
   const nsCString mAIALocation;
@@ -199,9 +203,8 @@ nsresult OCSPRequest::GetResponse(/*out*/ Vector<uint8_t>& response) {
   return NS_OK;
 }
 
-static NS_NAMED_LITERAL_CSTRING(OCSP_REQUEST_MIME_TYPE,
-                                "application/ocsp-request");
-static NS_NAMED_LITERAL_CSTRING(OCSP_REQUEST_METHOD, "POST");
+static constexpr auto OCSP_REQUEST_MIME_TYPE = "application/ocsp-request"_ns;
+static constexpr auto OCSP_REQUEST_METHOD = "POST"_ns;
 
 NS_IMETHODIMP
 OCSPRequest::Run() {
@@ -248,7 +251,7 @@ OCSPRequest::Run() {
                        nullptr,  // aLoadingNode
                        nsContentUtils::GetSystemPrincipal(),
                        nullptr,  // aTriggeringPrincipal
-                       nsILoadInfo::SEC_ALLOW_CROSS_ORIGIN_DATA_IS_NULL,
+                       nsILoadInfo::SEC_ALLOW_CROSS_ORIGIN_SEC_CONTEXT_IS_NULL,
                        nsIContentPolicy::TYPE_OTHER, getter_AddRefs(channel));
   if (NS_FAILED(rv)) {
     return NotifyDone(rv, lock);
@@ -261,9 +264,10 @@ OCSPRequest::Run() {
     priorityChannel->AdjustPriority(nsISupportsPriority::PRIORITY_HIGHEST);
   }
 
-  channel->SetLoadFlags(nsIRequest::LOAD_ANONYMOUS |
-                        nsIChannel::LOAD_BYPASS_SERVICE_WORKER |
-                        nsIChannel::LOAD_BYPASS_URL_CLASSIFIER);
+  channel->SetLoadFlags(
+      nsIRequest::LOAD_ANONYMOUS | nsIRequest::LOAD_BYPASS_CACHE |
+      nsIRequest::INHIBIT_CACHING | nsIChannel::LOAD_BYPASS_SERVICE_WORKER |
+      nsIChannel::LOAD_BYPASS_URL_CLASSIFIER);
 
   nsCOMPtr<nsILoadInfo> loadInfo = channel->LoadInfo();
 
@@ -307,7 +311,7 @@ OCSPRequest::Run() {
   if (NS_FAILED(rv)) {
     return NotifyDone(rv, lock);
   }
-  // Do not use SPDY for internal security operations. It could result
+  // Do not use SPDY or HTTP3 for internal security operations. It could result
   // in the silent upgrade to ssl, which in turn could require an SSL
   // operation to fulfill something like an OCSP fetch, which is an
   // endless loop.
@@ -319,6 +323,14 @@ OCSPRequest::Run() {
   if (NS_FAILED(rv)) {
     return NotifyDone(rv, lock);
   }
+  rv = internalChannel->SetAllowHttp3(false);
+  if (NS_FAILED(rv)) {
+    return NotifyDone(rv, lock);
+  }
+  rv = internalChannel->SetIsOCSP(true);
+  if (NS_FAILED(rv)) {
+    return NotifyDone(rv, lock);
+  }
   nsCOMPtr<nsIHttpChannel> hchan = do_QueryInterface(channel);
   if (!hchan) {
     return NotifyDone(NS_ERROR_FAILURE, lock);
@@ -487,49 +499,60 @@ mozilla::pkix::Result DoOCSPRequest(
   return Success;
 }
 
-static char* ShowProtectedAuthPrompt(PK11SlotInfo* slot,
-                                     nsIInterfaceRequestor* ir) {
-  if (!NS_IsMainThread()) {
-    NS_ERROR("ShowProtectedAuthPrompt called off the main thread");
+static char* ShowProtectedAuthPrompt(PK11SlotInfo* slot, nsIPrompt* prompt) {
+  MOZ_ASSERT(NS_IsMainThread());
+  MOZ_ASSERT(slot);
+  MOZ_ASSERT(prompt);
+  if (!NS_IsMainThread() || !slot || !prompt) {
     return nullptr;
   }
 
-  char* protAuthRetVal = nullptr;
-
-  // Get protected auth dialogs
-  nsCOMPtr<nsITokenDialogs> dialogs;
-  nsresult nsrv =
-      getNSSDialogs(getter_AddRefs(dialogs), NS_GET_IID(nsITokenDialogs),
-                    NS_TOKENDIALOGS_CONTRACTID);
-  if (NS_SUCCEEDED(nsrv)) {
-    RefPtr<nsProtectedAuthThread> protectedAuthRunnable =
-        new nsProtectedAuthThread();
-    protectedAuthRunnable->SetParams(slot);
-
-    nsrv = dialogs->DisplayProtectedAuth(ir, protectedAuthRunnable);
-
-    // We call join on the thread,
-    // so we can be sure that no simultaneous access will happen.
-    protectedAuthRunnable->Join();
-
-    if (NS_SUCCEEDED(nsrv)) {
-      SECStatus rv = protectedAuthRunnable->GetResult();
-      switch (rv) {
-        case SECSuccess:
-          protAuthRetVal =
-              ToNewCString(nsDependentCString(PK11_PW_AUTHENTICATED));
-          break;
-        case SECWouldBlock:
-          protAuthRetVal = ToNewCString(nsDependentCString(PK11_PW_RETRY));
-          break;
-        default:
-          protAuthRetVal = nullptr;
-          break;
-      }
-    }
+  // Dispatch a background task to (eventually) call C_Login. The call will
+  // block until the protected authentication succeeds or fails.
+  Atomic<bool> done;
+  Atomic<SECStatus> result;
+  nsresult rv =
+      NS_DispatchBackgroundTask(NS_NewRunnableFunction(__func__, [&]() mutable {
+        result = PK11_CheckUserPassword(slot, nullptr);
+        done = true;
+      }));
+  if (NS_FAILED(rv)) {
+    return nullptr;
   }
 
-  return protAuthRetVal;
+  nsTArray<nsCString> resIds = {
+      "security/pippki/pippki.ftl"_ns,
+  };
+  RefPtr<mozilla::intl::Localization> l10n =
+      mozilla::intl::Localization::Create(resIds, true);
+  auto l10nId = "protected-auth-alert"_ns;
+  auto l10nArgs = mozilla::dom::Optional<intl::L10nArgs>();
+  l10nArgs.Construct();
+  auto dirArg = l10nArgs.Value().Entries().AppendElement();
+  dirArg->mKey = "tokenName"_ns;
+  dirArg->mValue.SetValue().SetAsUTF8String().Assign(PK11_GetTokenName(slot));
+  nsAutoCString promptString;
+  ErrorResult errorResult;
+  l10n->FormatValueSync(l10nId, l10nArgs, promptString, errorResult);
+  if (NS_FAILED(errorResult.StealNSResult())) {
+    return nullptr;
+  }
+  rv = prompt->Alert(nullptr, NS_ConvertUTF8toUTF16(promptString).get());
+  if (NS_FAILED(rv)) {
+    return nullptr;
+  }
+
+  MOZ_ALWAYS_TRUE(SpinEventLoopUntil(
+      "ShowProtectedAuthPrompt"_ns, [&]() { return static_cast<bool>(done); }));
+
+  switch (result) {
+    case SECSuccess:
+      return ToNewCString(nsDependentCString(PK11_PW_AUTHENTICATED));
+    case SECWouldBlock:
+      return ToNewCString(nsDependentCString(PK11_PW_RETRY));
+    default:
+      return nullptr;
+  }
 }
 
 class PK11PasswordPromptRunnable : public SyncRunnableBase {
@@ -542,11 +565,32 @@ class PK11PasswordPromptRunnable : public SyncRunnableBase {
   virtual void RunOnTargetThread() override;
 
  private:
-  PK11SlotInfo* const mSlot;         // in
-  nsIInterfaceRequestor* const mIR;  // in
+  static bool mRunning;
+
+  PK11SlotInfo* mSlot;
+  nsIInterfaceRequestor* mIR;
 };
 
+bool PK11PasswordPromptRunnable::mRunning = false;
+
 void PK11PasswordPromptRunnable::RunOnTargetThread() {
+  MOZ_ASSERT(NS_IsMainThread());
+  if (!NS_IsMainThread()) {
+    return;
+  }
+
+  // If we've reentered due to the nested event loop implicit in using
+  // nsIPrompt synchronously (or indeed the explicit nested event loop in the
+  // protected authentication case), bail early, cancelling the password
+  // prompt. This will probably cause the operation that resulted in the prompt
+  // to fail, but this is better than littering the screen with a bunch of
+  // password prompts that the user will probably just cancel anyway.
+  if (mRunning) {
+    return;
+  }
+  mRunning = true;
+  auto setRunningToFalseOnExit = MakeScopeExit([&]() { mRunning = false; });
+
   nsresult rv;
   nsCOMPtr<nsIPrompt> prompt;
   if (!mIR) {
@@ -564,17 +608,17 @@ void PK11PasswordPromptRunnable::RunOnTargetThread() {
   }
 
   if (PK11_ProtectedAuthenticationPath(mSlot)) {
-    mResult = ShowProtectedAuthPrompt(mSlot, mIR);
+    mResult = ShowProtectedAuthPrompt(mSlot, prompt);
     return;
   }
 
   nsAutoString promptString;
   if (PK11_IsInternal(mSlot)) {
-    rv = GetPIPNSSBundleString("CertPassPromptDefault", promptString);
+    rv = GetPIPNSSBundleString("CertPasswordPromptDefault", promptString);
   } else {
     AutoTArray<nsString, 1> formatStrings = {
         NS_ConvertUTF8toUTF16(PK11_GetTokenName(mSlot))};
-    rv = PIPBundleFormatStringFromName("CertPassPrompt", formatStrings,
+    rv = PIPBundleFormatStringFromName("CertPasswordPrompt", formatStrings,
                                        promptString);
   }
   if (NS_FAILED(rv)) {
@@ -582,13 +626,9 @@ void PK11PasswordPromptRunnable::RunOnTargetThread() {
   }
 
   nsString password;
-  // |checkState| is unused because |checkMsg| (the argument just before it) is
-  // null, but XPConnect requires it to point to a valid bool nonetheless.
-  bool checkState = false;
   bool userClickedOK = false;
   rv = prompt->PromptPassword(nullptr, promptString.get(),
-                              getter_Copies(password), nullptr, &checkState,
-                              &userClickedOK);
+                              getter_Copies(password), &userClickedOK);
   if (NS_FAILED(rv) || !userClickedOK) {
     return;
   }
@@ -597,6 +637,9 @@ void PK11PasswordPromptRunnable::RunOnTargetThread() {
 }
 
 char* PK11PasswordPrompt(PK11SlotInfo* slot, PRBool /*retry*/, void* arg) {
+  if (!slot) {
+    return nullptr;
+  }
   RefPtr<PK11PasswordPromptRunnable> runnable(new PK11PasswordPromptRunnable(
       slot, static_cast<nsIInterfaceRequestor*>(arg)));
   runnable->DispatchToMainThreadAndWait();
@@ -607,35 +650,35 @@ nsCString getKeaGroupName(uint32_t aKeaGroup) {
   nsCString groupName;
   switch (aKeaGroup) {
     case ssl_grp_ec_secp256r1:
-      groupName = NS_LITERAL_CSTRING("P256");
+      groupName = "P256"_ns;
       break;
     case ssl_grp_ec_secp384r1:
-      groupName = NS_LITERAL_CSTRING("P384");
+      groupName = "P384"_ns;
       break;
     case ssl_grp_ec_secp521r1:
-      groupName = NS_LITERAL_CSTRING("P521");
+      groupName = "P521"_ns;
       break;
     case ssl_grp_ec_curve25519:
-      groupName = NS_LITERAL_CSTRING("x25519");
+      groupName = "x25519"_ns;
       break;
     case ssl_grp_ffdhe_2048:
-      groupName = NS_LITERAL_CSTRING("FF 2048");
+      groupName = "FF 2048"_ns;
       break;
     case ssl_grp_ffdhe_3072:
-      groupName = NS_LITERAL_CSTRING("FF 3072");
+      groupName = "FF 3072"_ns;
       break;
     case ssl_grp_none:
-      groupName = NS_LITERAL_CSTRING("none");
+      groupName = "none"_ns;
       break;
     case ssl_grp_ffdhe_custom:
-      groupName = NS_LITERAL_CSTRING("custom");
+      groupName = "custom"_ns;
       break;
     // All other groups are not enabled in Firefox. See namedGroups in
     // nsNSSIOLayer.cpp.
     default:
       // This really shouldn't happen!
       MOZ_ASSERT_UNREACHABLE("Invalid key exchange group.");
-      groupName = NS_LITERAL_CSTRING("unknown group");
+      groupName = "unknown group"_ns;
   }
   return groupName;
 }
@@ -644,91 +687,78 @@ nsCString getSignatureName(uint32_t aSignatureScheme) {
   nsCString signatureName;
   switch (aSignatureScheme) {
     case ssl_sig_none:
-      signatureName = NS_LITERAL_CSTRING("none");
+      signatureName = "none"_ns;
       break;
     case ssl_sig_rsa_pkcs1_sha1:
-      signatureName = NS_LITERAL_CSTRING("RSA-PKCS1-SHA1");
+      signatureName = "RSA-PKCS1-SHA1"_ns;
       break;
     case ssl_sig_rsa_pkcs1_sha256:
-      signatureName = NS_LITERAL_CSTRING("RSA-PKCS1-SHA256");
+      signatureName = "RSA-PKCS1-SHA256"_ns;
       break;
     case ssl_sig_rsa_pkcs1_sha384:
-      signatureName = NS_LITERAL_CSTRING("RSA-PKCS1-SHA384");
+      signatureName = "RSA-PKCS1-SHA384"_ns;
       break;
     case ssl_sig_rsa_pkcs1_sha512:
-      signatureName = NS_LITERAL_CSTRING("RSA-PKCS1-SHA512");
+      signatureName = "RSA-PKCS1-SHA512"_ns;
       break;
     case ssl_sig_ecdsa_secp256r1_sha256:
-      signatureName = NS_LITERAL_CSTRING("ECDSA-P256-SHA256");
+      signatureName = "ECDSA-P256-SHA256"_ns;
       break;
     case ssl_sig_ecdsa_secp384r1_sha384:
-      signatureName = NS_LITERAL_CSTRING("ECDSA-P384-SHA384");
+      signatureName = "ECDSA-P384-SHA384"_ns;
       break;
     case ssl_sig_ecdsa_secp521r1_sha512:
-      signatureName = NS_LITERAL_CSTRING("ECDSA-P521-SHA512");
+      signatureName = "ECDSA-P521-SHA512"_ns;
       break;
     case ssl_sig_rsa_pss_sha256:
-      signatureName = NS_LITERAL_CSTRING("RSA-PSS-SHA256");
+      signatureName = "RSA-PSS-SHA256"_ns;
       break;
     case ssl_sig_rsa_pss_sha384:
-      signatureName = NS_LITERAL_CSTRING("RSA-PSS-SHA384");
+      signatureName = "RSA-PSS-SHA384"_ns;
       break;
     case ssl_sig_rsa_pss_sha512:
-      signatureName = NS_LITERAL_CSTRING("RSA-PSS-SHA512");
+      signatureName = "RSA-PSS-SHA512"_ns;
       break;
     case ssl_sig_ecdsa_sha1:
-      signatureName = NS_LITERAL_CSTRING("ECDSA-SHA1");
+      signatureName = "ECDSA-SHA1"_ns;
       break;
     case ssl_sig_rsa_pkcs1_sha1md5:
-      signatureName = NS_LITERAL_CSTRING("RSA-PKCS1-SHA1MD5");
+      signatureName = "RSA-PKCS1-SHA1MD5"_ns;
       break;
     // All other groups are not enabled in Firefox. See sEnabledSignatureSchemes
     // in nsNSSIOLayer.cpp.
     default:
       // This really shouldn't happen!
       MOZ_ASSERT_UNREACHABLE("Invalid signature scheme.");
-      signatureName = NS_LITERAL_CSTRING("unknown signature");
+      signatureName = "unknown signature"_ns;
   }
   return signatureName;
 }
 
-// call with shutdown prevention lock held
 static void PreliminaryHandshakeDone(PRFileDesc* fd) {
-  nsNSSSocketInfo* infoObject = (nsNSSSocketInfo*)fd->higher->secret;
-  if (!infoObject) return;
-
-  SSLChannelInfo channelInfo;
-  if (SSL_GetChannelInfo(fd, &channelInfo, sizeof(channelInfo)) == SECSuccess) {
-    infoObject->SetSSLVersionUsed(channelInfo.protocolVersion);
-    infoObject->SetEarlyDataAccepted(channelInfo.earlyDataAccepted);
-    infoObject->SetResumed(channelInfo.resumed);
-
-    SSLCipherSuiteInfo cipherInfo;
-    if (SSL_GetCipherSuiteInfo(channelInfo.cipherSuite, &cipherInfo,
-                               sizeof cipherInfo) == SECSuccess) {
-      /* Set the Status information */
-      infoObject->mHaveCipherSuiteAndProtocol = true;
-      infoObject->mCipherSuite = channelInfo.cipherSuite;
-      infoObject->mProtocolVersion = channelInfo.protocolVersion & 0xFF;
-      infoObject->mKeaGroup.Assign(getKeaGroupName(channelInfo.keaGroup));
-      infoObject->mSignatureSchemeName.Assign(
-          getSignatureName(channelInfo.signatureScheme));
-      infoObject->SetKEAUsed(channelInfo.keaType);
-      infoObject->SetKEAKeyBits(channelInfo.keaKeyBits);
-      infoObject->SetMACAlgorithmUsed(cipherInfo.macAlgorithm);
-      infoObject->mIsDelegatedCredential = channelInfo.peerDelegCred;
-
-      if (infoObject->mIsDelegatedCredential) {
-        Telemetry::ScalarAdd(
-            Telemetry::ScalarID::SECURITY_TLS_DELEGATED_CREDENTIALS_TXN, 1);
-      }
-    }
-  }
-
-  // Don't update NPN details on renegotiation.
-  if (infoObject->IsPreliminaryHandshakeDone()) {
+  NSSSocketControl* socketControl = (NSSSocketControl*)fd->higher->secret;
+  if (!socketControl) {
     return;
   }
+  if (socketControl->IsPreliminaryHandshakeDone()) {
+    return;
+  }
+
+  SSLChannelInfo channelInfo;
+  if (SSL_GetChannelInfo(fd, &channelInfo, sizeof(channelInfo)) != SECSuccess) {
+    return;
+  }
+  SSLCipherSuiteInfo cipherInfo;
+  if (SSL_GetCipherSuiteInfo(channelInfo.cipherSuite, &cipherInfo,
+                             sizeof(cipherInfo)) != SECSuccess) {
+    return;
+  }
+  socketControl->SetPreliminaryHandshakeInfo(channelInfo, cipherInfo);
+  socketControl->SetSSLVersionUsed(channelInfo.protocolVersion);
+  socketControl->SetEarlyDataAccepted(channelInfo.earlyDataAccepted);
+  socketControl->SetKEAUsed(channelInfo.keaType);
+  socketControl->SetKEAKeyBits(channelInfo.keaKeyBits);
+  socketControl->SetMACAlgorithmUsed(cipherInfo.macAlgorithm);
 
   // Get the NPN value.
   SSLNextProtoState state;
@@ -740,24 +770,24 @@ static void PreliminaryHandshakeDone(PRFileDesc* fd) {
       SECSuccess) {
     if (state == SSL_NEXT_PROTO_NEGOTIATED ||
         state == SSL_NEXT_PROTO_SELECTED) {
-      infoObject->SetNegotiatedNPN(BitwiseCast<char*, unsigned char*>(npnbuf),
-                                   npnlen);
+      socketControl->SetNegotiatedNPN(
+          BitwiseCast<char*, unsigned char*>(npnbuf), npnlen);
     } else {
-      infoObject->SetNegotiatedNPN(nullptr, 0);
+      socketControl->SetNegotiatedNPN(nullptr, 0);
     }
     mozilla::Telemetry::Accumulate(Telemetry::SSL_NPN_TYPE, state);
   } else {
-    infoObject->SetNegotiatedNPN(nullptr, 0);
+    socketControl->SetNegotiatedNPN(nullptr, 0);
   }
 
-  infoObject->SetPreliminaryHandshakeDone();
+  socketControl->SetPreliminaryHandshakeDone();
 }
 
 SECStatus CanFalseStartCallback(PRFileDesc* fd, void* client_data,
                                 PRBool* canFalseStart) {
   *canFalseStart = false;
 
-  nsNSSSocketInfo* infoObject = (nsNSSSocketInfo*)fd->higher->secret;
+  NSSSocketControl* infoObject = (NSSSocketControl*)fd->higher->secret;
   if (!infoObject) {
     PR_SetError(PR_INVALID_STATE_ERROR, 0);
     return SECFailure;
@@ -836,48 +866,27 @@ SECStatus CanFalseStartCallback(PRFileDesc* fd, void* client_data,
 
 static void AccumulateNonECCKeySize(Telemetry::HistogramID probe,
                                     uint32_t bits) {
-  unsigned int value =
-      bits < 512
-          ? 1
-          : bits == 512
-                ? 2
-                : bits < 768
-                      ? 3
-                      : bits == 768
-                            ? 4
-                            : bits < 1024
-                                  ? 5
-                                  : bits == 1024
-                                        ? 6
-                                        : bits < 1280
-                                              ? 7
-                                              : bits == 1280
-                                                    ? 8
-                                                    : bits < 1536
-                                                          ? 9
-                                                          : bits == 1536
-                                                                ? 10
-                                                                : bits < 2048
-                                                                      ? 11
-                                                                      : bits == 2048
-                                                                            ? 12
-                                                                            : bits < 3072
-                                                                                  ? 13
-                                                                                  : bits == 3072
-                                                                                        ? 14
-                                                                                        : bits < 4096
-                                                                                              ? 15
-                                                                                              : bits == 4096
-                                                                                                    ? 16
-                                                                                                    : bits < 8192
-                                                                                                          ? 17
-                                                                                                          : bits == 8192
-                                                                                                                ? 18
-                                                                                                                : bits < 16384
-                                                                                                                      ? 19
-                                                                                                                      : bits == 16384
-                                                                                                                            ? 20
-                                                                                                                            : 0;
+  unsigned int value = bits < 512      ? 1
+                       : bits == 512   ? 2
+                       : bits < 768    ? 3
+                       : bits == 768   ? 4
+                       : bits < 1024   ? 5
+                       : bits == 1024  ? 6
+                       : bits < 1280   ? 7
+                       : bits == 1280  ? 8
+                       : bits < 1536   ? 9
+                       : bits == 1536  ? 10
+                       : bits < 2048   ? 11
+                       : bits == 2048  ? 12
+                       : bits < 3072   ? 13
+                       : bits == 3072  ? 14
+                       : bits < 4096   ? 15
+                       : bits == 4096  ? 16
+                       : bits < 8192   ? 17
+                       : bits == 8192  ? 18
+                       : bits < 16384  ? 19
+                       : bits == 16384 ? 20
+                                       : 0;
   Telemetry::Accumulate(probe, value);
 }
 
@@ -888,12 +897,11 @@ static void AccumulateNonECCKeySize(Telemetry::HistogramID probe,
 // named curves for a given size (e.g. secp256k1 vs. secp256r1). We punt on
 // that for now. See also NSS bug 323674.
 static void AccumulateECCCurve(Telemetry::HistogramID probe, uint32_t bits) {
-  unsigned int value =
-      bits == 255 ? 29                                            // Curve25519
-                  : bits == 256 ? 23                              // P-256
-                                : bits == 384 ? 24                // P-384
-                                              : bits == 521 ? 25  // P-521
-                                                            : 0;  // Unknown
+  unsigned int value = bits == 255   ? 29  // Curve25519
+                       : bits == 256 ? 23  // P-256
+                       : bits == 384 ? 24  // P-384
+                       : bits == 521 ? 25  // P-521
+                                     : 0;  // Unknown
   Telemetry::Accumulate(probe, value);
 }
 
@@ -1035,230 +1043,10 @@ static void AccumulateCipherSuite(Telemetry::HistogramID probe,
   Telemetry::Accumulate(probe, value);
 }
 
-// In the case of session resumption, the AuthCertificate hook has been bypassed
-// (because we've previously successfully connected to our peer). That being the
-// case, we unfortunately don't know what the verified certificate chain was, if
-// the peer's server certificate verified as extended validation, or what its CT
-// status is (if enabled). To address this, we attempt to build a certificate
-// chain here using as much of the original context as possible (e.g. stapled
-// OCSP responses, SCTs, the hostname, the first party domain, etc.). Note that
-// because we are on the socket thread, this must not cause any network
-// requests, hence the use of FLAG_LOCAL_ONLY.
-static void RebuildVerifiedCertificateInformation(PRFileDesc* fd,
-                                                  nsNSSSocketInfo* infoObject) {
-  MOZ_ASSERT(fd);
-  MOZ_ASSERT(infoObject);
-
-  if (!fd || !infoObject) {
-    return;
-  }
-
-  UniqueCERTCertificate cert(SSL_PeerCertificate(fd));
-  MOZ_ASSERT(cert, "SSL_PeerCertificate failed in TLS handshake callback?");
-  if (!cert) {
-    return;
-  }
-
-  Maybe<nsTArray<nsTArray<uint8_t>>> maybePeerCertsBytes;
-  UniqueCERTCertList peerCertChain(SSL_PeerCertificateChain(fd));
-  if (!peerCertChain) {
-    MOZ_LOG(gPIPNSSLog, LogLevel::Debug,
-            ("RebuildVerifiedCertificateInformation: failed to get peer "
-             "certificate chain"));
-  } else {
-    nsTArray<nsTArray<uint8_t>> peerCertsBytes;
-    for (CERTCertListNode* n = CERT_LIST_HEAD(peerCertChain);
-         !CERT_LIST_END(n, peerCertChain); n = CERT_LIST_NEXT(n)) {
-      // Don't include the end-entity certificate.
-      if (n == CERT_LIST_HEAD(peerCertChain)) {
-        continue;
-      }
-      nsTArray<uint8_t> certBytes;
-      certBytes.AppendElements(n->cert->derCert.data, n->cert->derCert.len);
-      peerCertsBytes.AppendElement(std::move(certBytes));
-    }
-    maybePeerCertsBytes.emplace(std::move(peerCertsBytes));
-  }
-
-  RefPtr<SharedCertVerifier> certVerifier(GetDefaultCertVerifier());
-  MOZ_ASSERT(certVerifier,
-             "Certificate verifier uninitialized in TLS handshake callback?");
-  if (!certVerifier) {
-    return;
-  }
-
-  // We don't own these pointers.
-  const SECItemArray* stapledOCSPResponses = SSL_PeerStapledOCSPResponses(fd);
-  Maybe<nsTArray<uint8_t>> stapledOCSPResponse;
-  // we currently only support single stapled responses
-  if (stapledOCSPResponses && stapledOCSPResponses->len == 1) {
-    stapledOCSPResponse.emplace();
-    stapledOCSPResponse->SetCapacity(stapledOCSPResponses->items[0].len);
-    stapledOCSPResponse->AppendElements(stapledOCSPResponses->items[0].data,
-                                        stapledOCSPResponses->items[0].len);
-  }
-
-  Maybe<nsTArray<uint8_t>> sctsFromTLSExtension;
-  const SECItem* sctsFromTLSExtensionSECItem = SSL_PeerSignedCertTimestamps(fd);
-  if (sctsFromTLSExtensionSECItem) {
-    sctsFromTLSExtension.emplace();
-    sctsFromTLSExtension->SetCapacity(sctsFromTLSExtensionSECItem->len);
-    sctsFromTLSExtension->AppendElements(sctsFromTLSExtensionSECItem->data,
-                                         sctsFromTLSExtensionSECItem->len);
-  }
-
-  int flags = mozilla::psm::CertVerifier::FLAG_LOCAL_ONLY;
-  if (!infoObject->SharedState().IsOCSPStaplingEnabled() ||
-      !infoObject->SharedState().IsOCSPMustStapleEnabled()) {
-    flags |= CertVerifier::FLAG_TLS_IGNORE_STATUS_REQUEST;
-  }
-
-  SECOidTag evOidPolicy;
-  CertificateTransparencyInfo certificateTransparencyInfo;
-  UniqueCERTCertList builtChain;
-  const bool saveIntermediates = false;
-  bool isBuiltCertChainRootBuiltInRoot = false;
-  mozilla::pkix::Result rv = certVerifier->VerifySSLServerCert(
-      cert, mozilla::pkix::Now(), infoObject, infoObject->GetHostName(),
-      builtChain, flags, maybePeerCertsBytes, stapledOCSPResponse,
-      sctsFromTLSExtension, Nothing(), infoObject->GetOriginAttributes(),
-      saveIntermediates, &evOidPolicy,
-      nullptr,  // OCSP stapling telemetry
-      nullptr,  // key size telemetry
-      nullptr,  // SHA-1 telemetry
-      nullptr,  // pinning telemetry
-      &certificateTransparencyInfo,
-      nullptr,  // CRLite telemetry,
-      &isBuiltCertChainRootBuiltInRoot);
-
-  if (rv != Success) {
-    MOZ_LOG(gPIPNSSLog, LogLevel::Debug,
-            ("HandshakeCallback: couldn't rebuild verified certificate info"));
-  }
-
-  RefPtr<nsNSSCertificate> nssc(nsNSSCertificate::Create(cert.get()));
-  if (rv == Success && evOidPolicy != SEC_OID_UNKNOWN) {
-    MOZ_LOG(gPIPNSSLog, LogLevel::Debug,
-            ("HandshakeCallback using NEW cert %p (is EV)", nssc.get()));
-    infoObject->SetServerCert(nssc, EVStatus::EV);
-  } else {
-    MOZ_LOG(gPIPNSSLog, LogLevel::Debug,
-            ("HandshakeCallback using NEW cert %p (is not EV)", nssc.get()));
-    infoObject->SetServerCert(nssc, EVStatus::NotEV);
-  }
-
-  if (rv == Success) {
-    uint16_t status =
-        TransportSecurityInfo::ConvertCertificateTransparencyInfoToStatus(
-            certificateTransparencyInfo);
-    infoObject->SetCertificateTransparencyStatus(status);
-    nsTArray<nsTArray<uint8_t>> certBytesArray =
-        TransportSecurityInfo::CreateCertBytesArray(builtChain);
-    infoObject->SetSucceededCertChain(std::move(certBytesArray));
-    infoObject->SetIsBuiltCertChainRootBuiltInRoot(
-        isBuiltCertChainRootBuiltInRoot);
-  }
-}
-
-nsresult IsCertificateDistrustImminent(
-    const nsTArray<RefPtr<nsIX509Cert>>& aCertArray,
-    /* out */ bool& isDistrusted) {
-  if (aCertArray.IsEmpty()) {
-    return NS_ERROR_INVALID_ARG;
-  }
-
-  nsCOMPtr<nsIX509Cert> rootCert;
-  nsTArray<RefPtr<nsIX509Cert>> intCerts;
-  nsCOMPtr<nsIX509Cert> eeCert;
-
-  nsresult rv = nsNSSCertificate::SegmentCertificateChain(aCertArray, rootCert,
-                                                          intCerts, eeCert);
-  if (NS_FAILED(rv)) {
-    return rv;
-  }
-
-  // Check the test certificate condition first; this is a special certificate
-  // that gets the 'imminent distrust' treatment; this is so that the distrust
-  // UX code does not become stale, as it will need regular use. See Bug 1409257
-  // for context. Please do not remove this when adjusting the rest of the
-  // method.
-  UniqueCERTCertificate nssEECert(eeCert->GetCert());
-  if (!nssEECert) {
-    return NS_ERROR_FAILURE;
-  }
-  isDistrusted =
-      CertDNIsInList(nssEECert.get(), TestImminentDistrustEndEntityDNs);
-  if (isDistrusted) {
-    // Exit early
-    return NS_OK;
-  }
-
-  UniqueCERTCertificate nssRootCert(rootCert->GetCert());
-  if (!nssRootCert) {
-    return NS_ERROR_FAILURE;
-  }
-
-  // Proceed with the Symantec imminent distrust algorithm. This algorithm is
-  // to be removed in Firefox 63, when the validity period check will also be
-  // removed from the code in NSSCertDBTrustDomain.
-  if (CertDNIsInList(nssRootCert.get(), RootSymantecDNs)) {
-    static const PRTime NULL_TIME = 0;
-
-    rv = CheckForSymantecDistrust(intCerts, eeCert, NULL_TIME,
-                                  RootAppleAndGoogleSPKIs, isDistrusted);
-    if (NS_FAILED(rv)) {
-      return rv;
-    }
-  }
-  return NS_OK;
-}
-
-static void RebuildCertificateInfoFromSSLTokenCache(
-    nsNSSSocketInfo* aInfoObject) {
-  MOZ_ASSERT(aInfoObject);
-
-  if (!aInfoObject) {
-    return;
-  }
-
-  nsAutoCString key;
-  aInfoObject->GetPeerId(key);
-  mozilla::net::SessionCacheInfo info;
-  if (!mozilla::net::SSLTokensCache::GetSessionCacheInfo(key, info)) {
-    MOZ_LOG(
-        gPIPNSSLog, LogLevel::Debug,
-        ("RebuildCertificateInfoFromSSLTokenCache cannot find cached info."));
-    return;
-  }
-
-  RefPtr<nsNSSCertificate> nssc = nsNSSCertificate::ConstructFromDER(
-      BitwiseCast<char*, uint8_t*>(info.mServerCertBytes.Elements()),
-      info.mServerCertBytes.Length());
-  if (!nssc) {
-    MOZ_LOG(gPIPNSSLog, LogLevel::Debug,
-            ("RebuildCertificateInfoFromSSLTokenCache failed to construct "
-             "server cert"));
-    return;
-  }
-
-  aInfoObject->SetServerCert(nssc, info.mEVStatus);
-  aInfoObject->SetCertificateTransparencyStatus(
-      info.mCertificateTransparencyStatus);
-  if (info.mSucceededCertChainBytes) {
-    aInfoObject->SetSucceededCertChain(
-        std::move(*info.mSucceededCertChainBytes));
-  }
-
-  if (info.mIsBuiltCertChainRootBuiltInRoot) {
-    aInfoObject->SetIsBuiltCertChainRootBuiltInRoot(
-        *info.mIsBuiltCertChainRootBuiltInRoot);
-  }
-}
-
 void HandshakeCallback(PRFileDesc* fd, void* client_data) {
   SECStatus rv;
 
-  nsNSSSocketInfo* infoObject = (nsNSSSocketInfo*)fd->higher->secret;
+  NSSSocketControl* infoObject = (NSSSocketControl*)fd->higher->secret;
 
   // Do the bookkeeping that needs to be done after the
   // server's ServerHello...ServerHelloDone have been processed, but that
@@ -1305,8 +1093,6 @@ void HandshakeCallback(PRFileDesc* fd, void* client_data) {
                                 : Telemetry::SSL_KEY_EXCHANGE_ALGORITHM_RESUMED,
                             channelInfo.keaType);
 
-      MOZ_ASSERT(infoObject->GetKEAUsed() == channelInfo.keaType);
-
       if (infoObject->IsFullHandshake()) {
         switch (channelInfo.keaType) {
           case ssl_kea_rsa:
@@ -1372,7 +1158,6 @@ void HandshakeCallback(PRFileDesc* fd, void* client_data) {
 
   bool deprecatedTlsVer =
       (channelInfo.protocolVersion < SSL_LIBRARY_VERSION_TLS_1_2);
-  RememberCertErrorsTable::GetInstance().LookupCertErrorBits(infoObject);
 
   uint32_t state;
   if (renegotiationUnsafe || deprecatedTlsVer) {
@@ -1392,39 +1177,11 @@ void HandshakeCallback(PRFileDesc* fd, void* client_data) {
     MOZ_LOG(gPIPNSSLog, LogLevel::Debug,
             ("HandshakeCallback KEEPING existing cert\n"));
   } else {
-    if (StaticPrefs::network_ssl_tokens_cache_enabled()) {
-      RebuildCertificateInfoFromSSLTokenCache(infoObject);
-      infoObject->NoteSessionResumptionTime(true);
-    } else {
-      RebuildVerifiedCertificateInformation(fd, infoObject);
-      infoObject->NoteSessionResumptionTime(false);
-    }
+    infoObject->RebuildCertificateInfoFromSSLTokenCache();
   }
 
-  nsTArray<RefPtr<nsIX509Cert>> succeededCertArray;
-  // The list could be empty. Bug 731478 will reduce the incidence of empty
-  // succeeded cert chains through better caching.
-  nsresult srv = infoObject->GetSucceededCertChain(succeededCertArray);
-
-  bool distrustImminent;
-  if (NS_SUCCEEDED(srv)) {
-    srv = IsCertificateDistrustImminent(succeededCertArray, distrustImminent);
-  }
-
-  if (NS_SUCCEEDED(srv) && distrustImminent) {
-    state |= nsIWebProgressListener::STATE_CERT_DISTRUST_IMMINENT;
-  }
-
-  bool domainMismatch;
-  bool untrusted;
-  bool notValidAtThisTime;
-  // These all return NS_OK, so don't even bother checking the return values.
-  Unused << infoObject->GetIsDomainMismatch(&domainMismatch);
-  Unused << infoObject->GetIsUntrusted(&untrusted);
-  Unused << infoObject->GetIsNotValidAtThisTime(&notValidAtThisTime);
-  // If we're here, the TLS handshake has succeeded. Thus if any of these
-  // booleans are true, the user has added an override for a certificate error.
-  if (domainMismatch || untrusted || notValidAtThisTime) {
+  // Check if the user has added an override for a certificate error.
+  if (infoObject->HasUserOverriddenCertificateError()) {
     state |= nsIWebProgressListener::STATE_CERT_USER_OVERRIDDEN;
   }
 
@@ -1440,7 +1197,7 @@ void HandshakeCallback(PRFileDesc* fd, void* client_data) {
     msg.AppendLiteral(" : server does not support RFC 5746, see CVE-2009-3555");
 
     nsContentUtils::LogSimpleConsoleError(
-        msg, "SSL", !!infoObject->GetOriginAttributes().mPrivateBrowsingId,
+        msg, "SSL"_ns, !!infoObject->GetOriginAttributes().mPrivateBrowsingId,
         true /* from chrome context */);
   }
 
