@@ -130,10 +130,10 @@ static bool IsGraphicsOkWithoutNetwork() {
       accessFlags = R_OK | W_OK;
     }
     if (access(socketPath.get(), accessFlags) != 0) {
-      SANDBOX_LOG_ERROR(
-          "%s is inaccessible (%s); can't isolate network namespace in"
+      SANDBOX_LOG_ERRNO(
+          "%s is inaccessible; can't isolate network namespace in"
           " content processes",
-          socketPath.get(), strerror(errno));
+          socketPath.get());
       return false;
     }
   }
@@ -265,6 +265,8 @@ static int GetEffectiveSandboxLevel(GeckoProcessType aType) {
       // GetEffectiveSocketProcessSandboxLevel is main-thread-only due to prefs.
       MOZ_ASSERT(NS_IsMainThread());
       return GetEffectiveSocketProcessSandboxLevel();
+    case GeckoProcessType_Utility:
+      return PR_GetEnv("MOZ_DISABLE_UTILITY_SANDBOX") == nullptr ? 1 : 0;
     default:
       return 0;
   }
@@ -326,7 +328,8 @@ void SandboxLaunchPrepare(GeckoProcessType aType,
     case GeckoProcessType_RDD:
       if (level >= 1) {
         canChroot = true;
-        flags |= CLONE_NEWNET | CLONE_NEWIPC;
+        // Can't use CLONE_NEWIPC because of intel-media-driver.
+        flags |= CLONE_NEWNET;
       }
       break;
     case GeckoProcessType_Content:
@@ -414,7 +417,7 @@ SandboxFork::SandboxFork(int aFlags, bool aChroot, int aServerFd, int aClientFd)
     int fds[2];
     int rv = socketpair(AF_UNIX, SOCK_STREAM | SOCK_CLOEXEC, 0, fds);
     if (rv != 0) {
-      SANDBOX_LOG_ERROR("socketpair: %s", strerror(errno));
+      SANDBOX_LOG_ERRNO("socketpair");
       MOZ_CRASH("socketpair failed");
     }
     mChrootClient = fds[0];
@@ -449,7 +452,7 @@ static void BlockAllSignals(sigset_t* aOldSigs) {
   MOZ_RELEASE_ASSERT(rv == 0);
   rv = pthread_sigmask(SIG_BLOCK, &allSigs, aOldSigs);
   if (rv != 0) {
-    SANDBOX_LOG_ERROR("pthread_sigmask (block all): %s", strerror(rv));
+    SANDBOX_LOG_WITH_ERROR(rv, "pthread_sigmask (block all)");
     MOZ_CRASH("pthread_sigmask");
   }
 }
@@ -460,13 +463,28 @@ static void RestoreSignals(const sigset_t* aOldSigs) {
   // state right now:
   int rv = pthread_sigmask(SIG_SETMASK, aOldSigs, nullptr);
   if (rv != 0) {
-    SANDBOX_LOG_ERROR("pthread_sigmask (restore): %s", strerror(-rv));
+    SANDBOX_LOG_WITH_ERROR(rv, "pthread_sigmask (restore)");
     MOZ_CRASH("pthread_sigmask");
   }
 }
 
+static bool IsSignalIgnored(int aSig) {
+  struct sigaction sa {};
+
+  if (sigaction(aSig, nullptr, &sa) != 0) {
+    if (errno != EINVAL) {
+      SANDBOX_LOG_ERRNO("sigaction(%d)", aSig);
+    }
+    return false;
+  }
+  return sa.sa_handler == SIG_IGN;
+}
+
 static void ResetSignalHandlers() {
   for (int signum = 1; signum <= SIGRTMAX; ++signum) {
+    if (IsSignalIgnored(signum)) {
+      continue;
+    }
     if (signal(signum, SIG_DFL) == SIG_ERR) {
       MOZ_DIAGNOSTIC_ASSERT(errno == EINVAL);
     }
@@ -506,11 +524,10 @@ static int CloneCallee(void* aPtr) {
 // Valgrind would disapprove of using clone() without CLONE_VM;
 // Chromium uses the raw syscall as a workaround in that case, but
 // we don't currently support sandboxing under valgrind.
-MOZ_NEVER_INLINE MOZ_ASAN_BLACKLIST static pid_t DoClone(int aFlags,
-                                                         jmp_buf* aCtx) {
+MOZ_NEVER_INLINE MOZ_ASAN_IGNORE static pid_t DoClone(int aFlags,
+                                                      jmp_buf* aCtx) {
   static constexpr size_t kStackAlignment = 16;
-  uint8_t miniStack[PTHREAD_STACK_MIN]
-      __attribute__((aligned(kStackAlignment)));
+  uint8_t miniStack[4096] __attribute__((aligned(kStackAlignment)));
 #ifdef __hppa__
   void* stackPtr = miniStack;
 #else
@@ -531,13 +548,19 @@ static pid_t ForkWithFlags(int aFlags) {
                                CLONE_CHILD_CLEARTID;
   MOZ_RELEASE_ASSERT((aFlags & kBadFlags) == 0);
 
+  // Block signals due to small stack in DoClone.
+  sigset_t oldSigs;
+  BlockAllSignals(&oldSigs);
+
+  int ret = 0;
   jmp_buf ctx;
   if (setjmp(ctx) == 0) {
     // In the parent and just called setjmp:
-    return DoClone(aFlags | SIGCHLD, &ctx);
+    ret = DoClone(aFlags | SIGCHLD, &ctx);
   }
+  RestoreSignals(&oldSigs);
   // In the child and have longjmp'ed:
-  return 0;
+  return ret;
 }
 
 static bool WriteStringToFile(const char* aPath, const char* aStr,
@@ -584,7 +607,7 @@ static void ConfigureUserNamespace(uid_t uid, gid_t gid) {
 
 static void DropAllCaps() {
   if (!LinuxCapabilities().SetCurrent()) {
-    SANDBOX_LOG_ERROR("capset (drop all): %s", strerror(errno));
+    SANDBOX_LOG_ERRNO("capset (drop all)");
   }
 }
 
@@ -648,7 +671,7 @@ void SandboxFork::StartChrootServer() {
   LinuxCapabilities caps;
   caps.Effective(CAP_SYS_CHROOT) = true;
   if (!caps.SetCurrent()) {
-    SANDBOX_LOG_ERROR("capset (chroot helper): %s", strerror(errno));
+    SANDBOX_LOG_ERRNO("capset (chroot helper)");
     MOZ_DIAGNOSTIC_ASSERT(false);
   }
 
