@@ -11,38 +11,32 @@
 
 #include "Sandbox.h"
 
+#include <CoreFoundation/CoreFoundation.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <CoreFoundation/CoreFoundation.h>
+#include <sys/sysctl.h>
+#include <sys/types.h>
+
 #include <iostream>
 #include <sstream>
 #include <vector>
 
-#include "mozilla/Assertions.h"
 #include "SandboxPolicyContent.h"
-#include "SandboxPolicyFlash.h"
 #include "SandboxPolicyGMP.h"
-#include "SandboxPolicyUtility.h"
+#include "SandboxPolicyRDD.h"
 #include "SandboxPolicySocket.h"
+#include "SandboxPolicyUtility.h"
+#include "mozilla/Assertions.h"
 
-// XXX There are currently problems with the /usr/include/sandbox.h file on
-// some/all of the Macs in Mozilla's build system. Further,
-// sandbox_init_with_parameters is not included in the header.  For the time
-// being (until this problem is resolved), we refer directly to what we need
-// from it, rather than including it here.
-extern "C" int sandbox_init(const char* profile, uint64_t flags, char** errorbuf);
+#include "mozilla/GeckoArgs.h"
+#include "mozilla/ipc/UtilityProcessSandboxing.h"
+
+// Undocumented sandbox setup routines.
 extern "C" int sandbox_init_with_parameters(const char* profile, uint64_t flags,
                                             const char* const parameters[], char** errorbuf);
 extern "C" void sandbox_free_error(char* errorbuf);
 extern "C" int sandbox_check(pid_t pid, const char* operation, int type, ...);
 
-#define MAC_OS_X_VERSION_10_0_HEX 0x00001000
-#define MAC_OS_X_VERSION_10_6_HEX 0x00001060
-#define MAC_OS_X_VERSION_10_7_HEX 0x00001070
-#define MAC_OS_X_VERSION_10_8_HEX 0x00001080
-#define MAC_OS_X_VERSION_10_9_HEX 0x00001090
-#define MAC_OS_X_VERSION_10_10_HEX 0x000010A0
-
 // Note about "major", "minor" and "bugfix" in the following code:
 //
 // The code decomposes an OS X version number into these components, and in
@@ -51,20 +45,32 @@ extern "C" int sandbox_check(pid_t pid, const char* operation, int type, ...);
 // an OS X version number to indicate a "major" release (for example the "9"
 // in OS X 10.9.5), and the "bugfix" component to indicate a "minor" release
 // (for example the "5" in OS X 10.9.5).
-
 class OSXVersion {
  public:
-  static int32_t OSXVersionMinor();
+  static void Get(int32_t& aMajor, int32_t& aMinor);
 
  private:
   static void GetSystemVersion(int32_t& aMajor, int32_t& aMinor, int32_t& aBugFix);
-  static int32_t GetVersionNumber();
-  static int32_t mOSXVersion;
+  static bool mCached;
+  static int32_t mOSXVersionMajor;
+  static int32_t mOSXVersionMinor;
 };
 
-int32_t OSXVersion::mOSXVersion = -1;
+bool OSXVersion::mCached = false;
+int32_t OSXVersion::mOSXVersionMajor;
+int32_t OSXVersion::mOSXVersionMinor;
 
-int32_t OSXVersion::OSXVersionMinor() { return (GetVersionNumber() & 0xF0) >> 4; }
+void OSXVersion::Get(int32_t& aMajor, int32_t& aMinor) {
+  if (!mCached) {
+    int32_t major, minor, bugfix;
+    GetSystemVersion(major, minor, bugfix);
+    mOSXVersionMajor = major;
+    mOSXVersionMinor = minor;
+    mCached = true;
+  }
+  aMajor = mOSXVersionMajor;
+  aMinor = mOSXVersionMinor;
+}
 
 void OSXVersion::GetSystemVersion(int32_t& aMajor, int32_t& aMinor, int32_t& aBugFix) {
   SInt32 major = 0, minor = 0, bugfix = 0;
@@ -99,12 +105,19 @@ void OSXVersion::GetSystemVersion(int32_t& aMajor, int32_t& aMinor, int32_t& aBu
   CFRelease(sysVersionPlist);
   CFRelease(versions);
 
-  // If 'major' isn't what we expect, assume the oldest version of OS X we
-  // currently support (OS X 10.6).
-  if (major != 10) {
+  if (major < 10) {
+    // If 'major' isn't what we expect, assume 10.6.
     aMajor = 10;
     aMinor = 6;
     aBugFix = 0;
+  } else if ((major == 10) && (minor >= 16)) {
+    // Account for SystemVersionCompat.plist being used which is
+    // automatically used for builds using older SDK versions and
+    // results in 11.0 being reported as 10.16. Assume the compat
+    // version will increase in step with the correct version.
+    aMajor = 11;
+    aMinor = minor - 16;
+    aBugFix = bugfix;
   } else {
     aMajor = major;
     aMinor = minor;
@@ -112,15 +125,6 @@ void OSXVersion::GetSystemVersion(int32_t& aMajor, int32_t& aMinor, int32_t& aBu
   }
 }
 
-int32_t OSXVersion::GetVersionNumber() {
-  if (mOSXVersion == -1) {
-    int32_t major, minor, bugfix;
-    GetSystemVersion(major, minor, bugfix);
-    mOSXVersion = MAC_OS_X_VERSION_10_0_HEX + (minor << 4) + bugfix;
-  }
-  return mOSXVersion;
-}
-
 bool GetRealPath(std::string& aOutputPath, const char* aInputPath) {
   char* resolvedPath = realpath(aInputPath, nullptr);
   if (resolvedPath == nullptr) {
@@ -133,6 +137,25 @@ bool GetRealPath(std::string& aOutputPath, const char* aInputPath) {
   return !aOutputPath.empty();
 }
 
+/*
+ * Returns true if the process is running under Rosetta translation. Returns
+ * false if running natively or if an error was encountered. To be called
+ * before enabling the sandbox therefore not requiring the sysctl be allowed
+ * by the sandbox policy. We use the `sysctl.proc_translated` sysctl which is
+ * documented by Apple to be used for this purpose.
+ */
+bool ProcessIsRosettaTranslated() {
+  int ret = 0;
+  size_t size = sizeof(ret);
+  if (sysctlbyname("sysctl.proc_translated", &ret, &size, NULL, 0) == -1) {
+    if (errno != ENOENT) {
+      fprintf(stderr, "Failed to check for translation environment\n");
+    }
+    return false;
+  }
+  return (ret == 1);
+}
+
 void MacSandboxInfo::AppendAsParams(std::vector<std::string>& aParams) const {
   this->AppendStartupParam(aParams);
   this->AppendLoggingParam(aParams);
@@ -148,8 +171,9 @@ void MacSandboxInfo::AppendAsParams(std::vector<std::string>& aParams) const {
       this->AppendDebugWriteDirParam(aParams);
 #endif
       break;
-    case MacSandboxType_Utility:
+    case MacSandboxType_RDD:
     case MacSandboxType_Socket:
+    case MacSandboxType_Utility:
       break;
     case MacSandboxType_GMP:
       this->AppendPluginPathParam(aParams);
@@ -245,57 +269,38 @@ namespace mozilla {
 bool StartMacSandbox(MacSandboxInfo const& aInfo, std::string& aErrorMessage) {
   std::vector<const char*> params;
   std::string profile;
-  std::string macOSMinor = std::to_string(OSXVersion::OSXVersionMinor());
 
-  // Used for the Flash sandbox. Declared here so that they
-  // stay in scope until sandbox_init_with_parameters is called.
-  std::string flashCacheDir, flashTempDir, flashPath;
+  // Use a combined version number to simplify version check logic
+  // in sandbox policies. For example, 10.14 becomes "1014".
+  int32_t major = 0, minor = 0;
+  OSXVersion::Get(major, minor);
+  MOZ_ASSERT(minor >= 0 && minor < 100);
+  std::string combinedVersion = std::to_string((major * 100) + minor);
 
-  if (aInfo.type == MacSandboxType_Flash) {
-    profile = SandboxPolicyFlash;
+  params.push_back("IS_ROSETTA_TRANSLATED");
+  params.push_back(ProcessIsRosettaTranslated() ? "TRUE" : "FALSE");
 
-    params.push_back("SHOULD_LOG");
-    params.push_back(aInfo.shouldLog ? "TRUE" : "FALSE");
+  // Used for the content process to access to parts of the cache dir.
+  std::string userCacheDir;
 
-    params.push_back("SANDBOX_LEVEL_1");
-    params.push_back(aInfo.level == 1 ? "TRUE" : "FALSE");
-    params.push_back("SANDBOX_LEVEL_2");
-    params.push_back(aInfo.level == 2 ? "TRUE" : "FALSE");
-
-    params.push_back("MAC_OS_MINOR");
-    params.push_back(macOSMinor.c_str());
-
-    params.push_back("HOME_PATH");
-    params.push_back(getenv("HOME"));
-
-    params.push_back("PLUGIN_BINARY_PATH");
-    if (!GetRealPath(flashPath, aInfo.pluginBinaryPath.c_str())) {
-      return false;
-    }
-    params.push_back(flashPath.c_str());
-
-    // User cache dir
-    params.push_back("DARWIN_USER_CACHE_DIR");
-    char confStrBuf[PATH_MAX];
-    if (!confstr(_CS_DARWIN_USER_CACHE_DIR, confStrBuf, sizeof(confStrBuf))) {
-      return false;
-    }
-    if (!GetRealPath(flashCacheDir, confStrBuf)) {
-      return false;
-    }
-    params.push_back(flashCacheDir.c_str());
-
-    // User temp dir
-    params.push_back("DARWIN_USER_TEMP_DIR");
-    if (!confstr(_CS_DARWIN_USER_TEMP_DIR, confStrBuf, sizeof(confStrBuf))) {
-      return false;
-    }
-    if (!GetRealPath(flashTempDir, confStrBuf)) {
-      return false;
-    }
-    params.push_back(flashTempDir.c_str());
-  } else if (aInfo.type == MacSandboxType_Utility) {
+  if (aInfo.type == MacSandboxType_Utility) {
     profile = const_cast<char*>(SandboxPolicyUtility);
+
+    switch (aInfo.utilityKind) {
+      case ipc::SandboxingKind::GENERIC_UTILITY:
+        // Nothing to do here specifically
+        break;
+
+      case ipc::SandboxingKind::UTILITY_AUDIO_DECODING_APPLE_MEDIA: {
+        profile.append(SandboxPolicyUtilityAudioDecoderAppleMediaAddend);
+        params.push_back("MAC_OS_VERSION");
+        params.push_back(combinedVersion.c_str());
+      } break;
+
+      default:
+        MOZ_ASSERT(false, "Invalid SandboxingKind");
+        break;
+    }
     params.push_back("SHOULD_LOG");
     params.push_back(aInfo.shouldLog ? "TRUE" : "FALSE");
     params.push_back("APP_PATH");
@@ -304,6 +309,20 @@ bool StartMacSandbox(MacSandboxInfo const& aInfo, std::string& aErrorMessage) {
       params.push_back("CRASH_PORT");
       params.push_back(aInfo.crashServerPort.c_str());
     }
+  } else if (aInfo.type == MacSandboxType_RDD) {
+    profile = const_cast<char*>(SandboxPolicyRDD);
+    params.push_back("SHOULD_LOG");
+    params.push_back(aInfo.shouldLog ? "TRUE" : "FALSE");
+    params.push_back("MAC_OS_VERSION");
+    params.push_back(combinedVersion.c_str());
+    params.push_back("APP_PATH");
+    params.push_back(aInfo.appPath.c_str());
+    params.push_back("HOME_PATH");
+    params.push_back(getenv("HOME"));
+    if (!aInfo.crashServerPort.empty()) {
+      params.push_back("CRASH_PORT");
+      params.push_back(aInfo.crashServerPort.c_str());
+    }
   } else if (aInfo.type == MacSandboxType_Socket) {
     profile = const_cast<char*>(SandboxPolicySocket);
     params.push_back("SHOULD_LOG");
@@ -354,8 +373,8 @@ bool StartMacSandbox(MacSandboxInfo const& aInfo, std::string& aErrorMessage) {
       params.push_back(aInfo.level == 2 ? "TRUE" : "FALSE");
       params.push_back("SANDBOX_LEVEL_3");
       params.push_back(aInfo.level == 3 ? "TRUE" : "FALSE");
-      params.push_back("MAC_OS_MINOR");
-      params.push_back(macOSMinor.c_str());
+      params.push_back("MAC_OS_VERSION");
+      params.push_back(combinedVersion.c_str());
       params.push_back("APP_PATH");
       params.push_back(aInfo.appPath.c_str());
       params.push_back("PROFILE_DIR");
@@ -370,6 +389,17 @@ bool StartMacSandbox(MacSandboxInfo const& aInfo, std::string& aErrorMessage) {
         params.push_back("CRASH_PORT");
         params.push_back(aInfo.crashServerPort.c_str());
       }
+
+      params.push_back("DARWIN_USER_CACHE_DIR");
+      char confStrBuf[PATH_MAX];
+      if (!confstr(_CS_DARWIN_USER_CACHE_DIR, confStrBuf, sizeof(confStrBuf))) {
+        return false;
+      }
+      if (!GetRealPath(userCacheDir, confStrBuf)) {
+        return false;
+      }
+      params.push_back(userCacheDir.c_str());
+
       if (!aInfo.testingReadPath1.empty()) {
         params.push_back("TESTING_READ_PATH1");
         params.push_back(aInfo.testingReadPath1.c_str());
@@ -574,7 +604,8 @@ bool GetContentSandboxParamsFromArgs(int aArgc, char** aArgv, MacSandboxInfo& aI
   return true;
 }
 
-bool GetUtilitySandboxParamsFromArgs(int aArgc, char** aArgv, MacSandboxInfo& aInfo) {
+bool GetUtilitySandboxParamsFromArgs(int aArgc, char** aArgv, MacSandboxInfo& aInfo,
+                                     bool aSandboxingKindRequired = true) {
   // Ensure we find these paramaters in the command
   // line arguments. Return false if any are missing.
   bool foundAppPath = false;
@@ -600,6 +631,16 @@ bool GetUtilitySandboxParamsFromArgs(int aArgc, char** aArgv, MacSandboxInfo& aI
     }
   }
 
+  if (aSandboxingKindRequired) {
+    Maybe<uint64_t> sandboxingKind =
+        geckoargs::sSandboxingKind.Get(aArgc, aArgv, CheckArgFlag::None);
+    if (sandboxingKind.isNothing()) {
+      fprintf(stderr, "Utility sandbox requires a sandboxingKind");
+      return false;
+    }
+    aInfo.utilityKind = (ipc::SandboxingKind)*sandboxingKind;
+  }
+
   if (!foundAppPath) {
     fprintf(stderr, "Utility sandbox disabled due to "
                     "missing sandbox CLI app path parameter.\n");
@@ -610,7 +651,7 @@ bool GetUtilitySandboxParamsFromArgs(int aArgc, char** aArgv, MacSandboxInfo& aI
 }
 
 bool GetSocketSandboxParamsFromArgs(int aArgc, char** aArgv, MacSandboxInfo& aInfo) {
-  return GetUtilitySandboxParamsFromArgs(aArgc, aArgv, aInfo);
+  return GetUtilitySandboxParamsFromArgs(aArgc, aArgv, aInfo, false);
 }
 
 bool GetPluginSandboxParamsFromArgs(int aArgc, char** aArgv, MacSandboxInfo& aInfo) {
@@ -684,6 +725,10 @@ bool GetPluginSandboxParamsFromArgs(int aArgc, char** aArgv, MacSandboxInfo& aIn
   return true;
 }
 
+bool GetRDDSandboxParamsFromArgs(int aArgc, char** aArgv, MacSandboxInfo& aInfo) {
+  return GetUtilitySandboxParamsFromArgs(aArgc, aArgv, aInfo, false);
+}
+
 /*
  * Returns true if no errors were encountered or if early sandbox startup is
  * not enabled for this process. Returns false if an error was encountered.
@@ -719,8 +764,13 @@ bool StartMacSandboxIfEnabled(const MacSandboxType aSandboxType, int aArgc, char
         return false;
       }
       break;
-    case MacSandboxType_Utility:
-      if (!GetUtilitySandboxParamsFromArgs(aArgc, aArgv, info)) {
+    case MacSandboxType_GMP:
+      if (!GetPluginSandboxParamsFromArgs(aArgc, aArgv, info)) {
+        return false;
+      }
+      break;
+    case MacSandboxType_RDD:
+      if (!GetRDDSandboxParamsFromArgs(aArgc, aArgv, info)) {
         return false;
       }
       break;
@@ -729,8 +779,8 @@ bool StartMacSandboxIfEnabled(const MacSandboxType aSandboxType, int aArgc, char
         return false;
       }
       break;
-    case MacSandboxType_GMP:
-      if (!GetPluginSandboxParamsFromArgs(aArgc, aArgv, info)) {
+    case MacSandboxType_Utility:
+      if (!GetUtilitySandboxParamsFromArgs(aArgc, aArgv, info)) {
         return false;
       }
       break;
