@@ -16,6 +16,7 @@
 #include "nsTArray.h"
 #include "mozilla/Maybe.h"
 #include "mozilla/widget/ThemeChangeKind.h"
+#include "mozilla/ColorScheme.h"
 
 struct gfxFontStyle;
 
@@ -23,6 +24,9 @@ class nsIFrame;
 
 namespace mozilla {
 
+using Modifiers = uint16_t;
+struct StyleColorSchemeFlags;
+
 namespace dom {
 class Document;
 }
@@ -38,12 +42,15 @@ enum class StyleSystemFont : uint8_t;
 class LookAndFeel {
  public:
   using ColorID = StyleSystemColor;
+  using ColorScheme = mozilla::ColorScheme;
 
   // When modifying this list, also modify nsXPLookAndFeel::sIntPrefs
   // in widget/xpwidgts/nsXPLookAndFeel.cpp.
   enum class IntID {
     // default, may be overriden by OS
     CaretBlinkTime,
+    // Amount of blinks that happen before the caret stops blinking.
+    CaretBlinkCount,
     // pixel width of caret
     CaretWidth,
     // show the caret when text is selected?
@@ -58,8 +65,6 @@ class LookAndFeel {
     UseOverlayScrollbars,
     // allow H and V overlay scrollbars to overlap?
     AllowOverlayScrollbarsOverlap,
-    // show/hide scrollbars based on activity
-    ShowHideScrollbars,
     // skip navigating to disabled menu item?
     SkipNavigatingDisabledMenuItem,
     // begin a drag if the mouse is moved further than the threshold while the
@@ -71,8 +76,6 @@ class LookAndFeel {
 
     // position of scroll arrows in a scrollbar
     ScrollArrowStyle,
-    // is scroll thumb proportional or fixed?
-    ScrollSliderStyle,
 
     // each button can take one of four values:
     ScrollButtonLeftMouseButtonAction,
@@ -143,22 +146,20 @@ class LookAndFeel {
     /*
      * A Boolean value to determine whether the Mac graphite theme is
      * being used.
-     *
-     * The value of this metric is not used on other platforms. These platforms
-     * should return NS_ERROR_NOT_IMPLEMENTED when queried for this metric.
      */
     MacGraphiteTheme,
 
     /*
      * A Boolean value to determine whether the macOS Big Sur-specific
      * theming should be used.
-     *
-     * The value of this metric is not used on non-Mac platforms. These
-     * platforms should return NS_ERROR_NOT_IMPLEMENTED when queried for this
-     * metric.
      */
     MacBigSurTheme,
 
+    /*
+     * A Boolean value to determine whether macOS is in RTL mode or not.
+     */
+    MacRTL,
+
     /*
      * AlertNotificationOrigin indicates from which corner of the
      * screen alerts slide in, and from which direction (horizontal/vertical).
@@ -198,14 +199,6 @@ class LookAndFeel {
      * If this metric != 0, support window dragging on the menubar.
      */
     MenuBarDrag,
-    /**
-     * Return the appropriate WindowsThemeIdentifier for the current theme.
-     */
-    WindowsThemeIdentifier,
-    /**
-     * Return an appropriate os version identifier.
-     */
-    OperatingSystemVersionIdentifier,
     /**
      * 0: scrollbar button repeats to scroll only when cursor is on the button.
      * 1: scrollbar button repeats to scroll even if cursor is outside of it.
@@ -216,8 +209,7 @@ class LookAndFeel {
      */
     TooltipDelay,
     /*
-     * A Boolean value to determine whether Mac OS X Lion style swipe animations
-     * should be used.
+     * A Boolean value to determine whether swipe animations should be used.
      */
     SwipeAnimationEnabled,
 
@@ -246,18 +238,6 @@ class LookAndFeel {
      */
     GTKCSDAvailable,
 
-    /*
-     * A boolean value indicating whether GTK+ system titlebar should be
-     * disabled by default.
-     */
-    GTKCSDHideTitlebarByDefault,
-
-    /*
-     * A boolean value indicating whether client-side decorations should
-     * have transparent background.
-     */
-    GTKCSDTransparentBackground,
-
     /*
      * A boolean value indicating whether client-side decorations should
      * contain a minimize button.
@@ -278,25 +258,19 @@ class LookAndFeel {
 
     /**
      * An Integer value that will represent the position of the Minimize button
-     * in GTK Client side decoration header. Its value will be between 0 and 2
-     * if it is on the left side of the tabbar, otherwise it will be between
-     * 3 and 5.
+     * in GTK Client side decoration header.
      */
     GTKCSDMinimizeButtonPosition,
 
     /**
      * An Integer value that will represent the position of the Maximize button
-     * in GTK Client side decoration header. Its value will be between 0 and 2
-     * if it is on the left side of the tabbar, otherwise it will be between
-     * 3 and 5.
+     * in GTK Client side decoration header.
      */
     GTKCSDMaximizeButtonPosition,
 
     /**
      * An Integer value that will represent the position of the Close button
-     * in GTK Client side decoration header. Its value will be between 0 and 2
-     * if it is on the left side of the tabbar, otherwise it will be between
-     * 3 and 5.
+     * in GTK Client side decoration header.
      */
     GTKCSDCloseButtonPosition,
 
@@ -318,8 +292,24 @@ class LookAndFeel {
      * 0: no-preference
      * 1: reduce
      */
-
     PrefersReducedMotion,
+
+    /**
+     * Corresponding to prefers-reduced-transparency.
+     * https://drafts.csswg.org/mediaqueries-5/#prefers-reduced-transparency
+     * 0: no-preference
+     * 1: reduce
+     */
+    PrefersReducedTransparency,
+
+    /**
+     * Corresponding to inverted-colors.
+     * https://drafts.csswg.org/mediaqueries-5/#inverted
+     * 0: none
+     * 1: inverted
+     */
+    InvertedColors,
+
     /**
      * Corresponding to PointerCapabilities in ServoTypes.h
      * 0: None
@@ -334,11 +324,27 @@ class LookAndFeel {
      * 'Coarse | Fine | Hover'.
      */
     AllPointerCapabilities,
-    /** The vertical scrollbar width, in CSS pixels. */
-    SystemVerticalScrollbarWidth,
 
-    /** The horizontal scrollbar height, in CSS pixels. */
-    SystemHorizontalScrollbarHeight,
+    /** The scrollbar size, in CSS pixels. */
+    SystemScrollbarSize,
+
+    /** A boolean value to determine whether a touch device is present */
+    TouchDeviceSupportPresent,
+
+    /** GTK titlebar radius */
+    TitlebarRadius,
+
+    /**
+     * Corresponding to dynamic-range.
+     * https://drafts.csswg.org/mediaqueries-5/#dynamic-range
+     * 0: Standard
+     * 1: High
+     */
+    DynamicRange,
+    VideoDynamicRange,
+
+    /** Whether XUL panel animations are enabled. */
+    PanelAnimations,
 
     /*
      * Not an ID; used to define the range of valid IDs.  Must be last.
@@ -351,30 +357,11 @@ class LookAndFeel {
     return GetInt(IntID::UseOverlayScrollbars);
   }
 
-  /**
-   * Windows themes we currently detect.
-   */
-  enum WindowsTheme {
-    eWindowsTheme_Generic = 0,  // unrecognized theme
-    eWindowsTheme_Classic,
-    eWindowsTheme_Aero,
-    eWindowsTheme_LunaBlue,
-    eWindowsTheme_LunaOlive,
-    eWindowsTheme_LunaSilver,
-    eWindowsTheme_Royale,
-    eWindowsTheme_Zune,
-    eWindowsTheme_AeroLite
-  };
-
-  /**
-   * Operating system versions.
-   */
-  enum class OperatingSystemVersion {
-    Windows7 = 2,
-    Windows8,
-    Windows10,
-    Unknown
-  };
+  // Returns keyCode value of a modifier key which is used for accesskey.
+  // Returns 0 if the platform doesn't support access key.
+  static uint32_t GetMenuAccessKey();
+  // Modifier mask for the menu accesskey.
+  static Modifiers GetMenuAccessKeyModifiers();
 
   enum {
     eScrollArrow_None = 0,
@@ -400,8 +387,6 @@ class LookAndFeel {
         eScrollArrow_StartBackward | eScrollArrow_StartForward
   };
 
-  enum { eScrollThumbStyle_Normal, eScrollThumbStyle_Proportional };
-
   // When modifying this list, also modify nsXPLookAndFeel::sFloatPrefs
   // in widget/nsXPLookAndFeel.cpp.
   enum class FloatID {
@@ -424,17 +409,33 @@ class LookAndFeel {
 
   using FontID = mozilla::StyleSystemFont;
 
-  // Whether we should use a light or dark appearance.
-  //
-  // This is currently ignored (but won't be for long).
-  enum class ColorScheme : uint8_t { Light, Dark };
+  static bool WindowsNonNativeMenusEnabled();
 
   static ColorScheme SystemColorScheme() {
     return GetInt(IntID::SystemUsesDarkTheme) ? ColorScheme::Dark
                                               : ColorScheme::Light;
   }
 
-  static ColorScheme ColorSchemeForDocument(const dom::Document& aDoc);
+  static bool IsDarkColor(nscolor);
+
+  enum class ChromeColorSchemeSetting { Light, Dark, System };
+  static ChromeColorSchemeSetting ColorSchemeSettingForChrome();
+  static ColorScheme ThemeDerivedColorSchemeForContent();
+
+  static ColorScheme ColorSchemeForChrome() {
+    MOZ_ASSERT(sColorSchemeInitialized);
+    return sChromeColorScheme;
+  }
+  static ColorScheme PreferredColorSchemeForContent() {
+    MOZ_ASSERT(sColorSchemeInitialized);
+    return sContentColorScheme;
+  }
+
+  static ColorScheme ColorSchemeForStyle(
+      const dom::Document&, const StyleColorSchemeFlags&,
+      ColorSchemeMode = ColorSchemeMode::Used);
+  static ColorScheme ColorSchemeForFrame(
+      const nsIFrame*, ColorSchemeMode = ColorSchemeMode::Used);
 
   // Whether standins for native colors should be used (that is, colors faked,
   // taken from win7, mostly). This forces light appearance, effectively.
@@ -452,16 +453,8 @@ class LookAndFeel {
   //   color value.
   static Maybe<nscolor> GetColor(ColorID, ColorScheme, UseStandins);
 
-  // Gets the color with appropriate defaults for UseStandins, ColorScheme etc
-  // for a given document.
-  static Maybe<nscolor> GetColor(ColorID, const dom::Document&);
-
   // Gets the color with appropriate defaults for UseStandins, ColorScheme etc
   // for a given frame.
-  //
-  // TODO(emilio): This right now just peeks the document out of the frame's
-  // pres context, but in the future we actually want to look at the style to
-  // get the right color scheme, to implement the color-scheme property.
   static Maybe<nscolor> GetColor(ColorID, const nsIFrame*);
 
   // Versions of the above which returns the color if found, or a default (which
@@ -472,16 +465,26 @@ class LookAndFeel {
     return GetColor(aId, aScheme, aUseStandins).valueOr(aDefault);
   }
 
-  static nscolor Color(ColorID aId, const dom::Document& aDoc,
-                       nscolor aDefault = NS_RGB(0, 0, 0)) {
-    return GetColor(aId, aDoc).valueOr(aDefault);
-  }
-
   static nscolor Color(ColorID aId, nsIFrame* aFrame,
                        nscolor aDefault = NS_RGB(0, 0, 0)) {
     return GetColor(aId, aFrame).valueOr(aDefault);
   }
 
+  static float GetTextScaleFactor() {
+    float f = GetFloat(FloatID::TextScaleFactor, 1.0f);
+    if (MOZ_UNLIKELY(f <= 0.0f)) {
+      return 1.0f;
+    }
+    return f;
+  }
+
+  struct ZoomSettings {
+    float mFullZoom = 1.0f;
+    float mTextZoom = 1.0f;
+  };
+
+  static ZoomSettings SystemZoomSettings();
+
   /**
    * GetInt() and GetFloat() return a int or float value for aID.  The result
    * might be distance, time, some flags or a int value which has particular
@@ -535,12 +538,20 @@ class LookAndFeel {
    */
   static bool GetEchoPassword();
 
+  /**
+   * Whether we should be drawing in the titlebar by default.
+   */
+  static bool DrawInTitlebar();
+
   /**
    * The millisecond to mask password value.
    * This value is only valid when GetEchoPassword() returns true.
    */
   static uint32_t GetPasswordMaskDelay();
 
+  /** Gets theme information for about:support */
+  static void GetThemeInfo(nsACString&);
+
   /**
    * When system look and feel is changed, Refresh() must be called.  Then,
    * cached data would be released.
@@ -559,6 +570,31 @@ class LookAndFeel {
 
   static void SetData(widget::FullLookAndFeel&& aTables);
   static void NotifyChangedAllWindows(widget::ThemeChangeKind);
+  static bool HasPendingGlobalThemeChange() { return sGlobalThemeChanged; }
+  static void HandleGlobalThemeChange() {
+    if (MOZ_UNLIKELY(HasPendingGlobalThemeChange())) {
+      DoHandleGlobalThemeChange();
+    }
+  }
+  static void EnsureColorSchemesInitialized() {
+    if (!sColorSchemeInitialized) {
+      RecomputeColorSchemes();
+    }
+    MOZ_ASSERT(sColorSchemeInitialized);
+  }
+
+  static ColorScheme sChromeColorScheme;
+  static ColorScheme sContentColorScheme;
+
+ protected:
+  static void RecomputeColorSchemes();
+  static bool sColorSchemeInitialized;
+
+  static void DoHandleGlobalThemeChange();
+  // Set to true when ThemeChanged needs to be called on mTheme (and other
+  // global LookAndFeel.  This is used because mTheme is a service, so there's
+  // no need to notify it from more than one prescontext.
+  static bool sGlobalThemeChanged;
 };
 
 }  // namespace mozilla
