@@ -19,6 +19,7 @@
 #include "mozilla/ScopeExit.h"
 #include "mozilla/dom/ScriptSettings.h"
 #include "mozilla/java/EventCallbackWrappers.h"
+#include "mozilla/jni/GeckoBundleUtils.h"
 
 // Disable the C++ 2a warning. See bug #1509926
 #if defined(__clang__)
@@ -38,266 +39,10 @@ bool CheckJS(JSContext* aCx, bool aResult) {
   return aResult;
 }
 
-nsresult BoxString(JSContext* aCx, JS::HandleValue aData,
-                   jni::Object::LocalRef& aOut) {
-  if (aData.isNullOrUndefined()) {
-    aOut = nullptr;
-    return NS_OK;
-  }
-
-  MOZ_ASSERT(aData.isString());
-
-  JS::RootedString str(aCx, aData.toString());
-
-  if (JS::StringHasLatin1Chars(str)) {
-    nsAutoJSString autoStr;
-    NS_ENSURE_TRUE(CheckJS(aCx, autoStr.init(aCx, str)), NS_ERROR_FAILURE);
-
-    // StringParam can automatically convert a nsString to jstring.
-    aOut = jni::StringParam(autoStr, aOut.Env(), fallible);
-    if (!aOut) {
-      return NS_ERROR_FAILURE;
-    }
-    return NS_OK;
-  }
-
-  // Two-byte string
-  JNIEnv* const env = aOut.Env();
-  const char16_t* chars;
-  {
-    JS::AutoCheckCannotGC nogc;
-    size_t len = 0;
-    chars = JS_GetTwoByteStringCharsAndLength(aCx, nogc, str, &len);
-    if (chars) {
-      aOut = jni::String::LocalRef::Adopt(
-          env, env->NewString(reinterpret_cast<const jchar*>(chars), len));
-    }
-  }
-  if (NS_WARN_IF(!CheckJS(aCx, !!chars) || !aOut)) {
-    env->ExceptionClear();
-    return NS_ERROR_FAILURE;
-  }
-  return NS_OK;
-}
-
-nsresult BoxObject(JSContext* aCx, JS::HandleValue aData,
-                   jni::Object::LocalRef& aOut);
-
-template <typename Type, bool (JS::Value::*IsType)() const,
-          Type (JS::Value::*ToType)() const, class ArrayType,
-          typename ArrayType::LocalRef (*NewArray)(const Type*, size_t)>
-nsresult BoxArrayPrimitive(JSContext* aCx, JS::HandleObject aData,
-                           jni::Object::LocalRef& aOut, size_t aLength,
-                           JS::HandleValue aElement) {
-  JS::RootedValue element(aCx);
-  auto data = MakeUnique<Type[]>(aLength);
-  data[0] = (aElement.get().*ToType)();
-
-  for (size_t i = 1; i < aLength; i++) {
-    NS_ENSURE_TRUE(CheckJS(aCx, JS_GetElement(aCx, aData, i, &element)),
-                   NS_ERROR_FAILURE);
-    NS_ENSURE_TRUE((element.get().*IsType)(), NS_ERROR_INVALID_ARG);
-
-    data[i] = (element.get().*ToType)();
-  }
-  aOut = (*NewArray)(data.get(), aLength);
-  return NS_OK;
-}
-
-template <class Type,
-          nsresult (*Box)(JSContext*, JS::HandleValue, jni::Object::LocalRef&),
-          typename IsType>
-nsresult BoxArrayObject(JSContext* aCx, JS::HandleObject aData,
-                        jni::Object::LocalRef& aOut, size_t aLength,
-                        JS::HandleValue aElement, IsType&& aIsType) {
-  auto out = jni::ObjectArray::New<Type>(aLength);
-  JS::RootedValue element(aCx);
-  jni::Object::LocalRef jniElement(aOut.Env());
-
-  nsresult rv = (*Box)(aCx, aElement, jniElement);
-  NS_ENSURE_SUCCESS(rv, rv);
-  out->SetElement(0, jniElement);
-
-  for (size_t i = 1; i < aLength; i++) {
-    NS_ENSURE_TRUE(CheckJS(aCx, JS_GetElement(aCx, aData, i, &element)),
-                   NS_ERROR_FAILURE);
-    NS_ENSURE_TRUE(element.isNullOrUndefined() || aIsType(element),
-                   NS_ERROR_INVALID_ARG);
-
-    rv = (*Box)(aCx, element, jniElement);
-    NS_ENSURE_SUCCESS(rv, rv);
-    out->SetElement(i, jniElement);
-  }
-  aOut = out;
-  return NS_OK;
-}
-
-nsresult BoxArray(JSContext* aCx, JS::HandleObject aData,
-                  jni::Object::LocalRef& aOut) {
-  uint32_t length = 0;
-  NS_ENSURE_TRUE(CheckJS(aCx, JS::GetArrayLength(aCx, aData, &length)),
-                 NS_ERROR_FAILURE);
-
-  if (!length) {
-    // Always represent empty arrays as an empty boolean array.
-    aOut = java::GeckoBundle::EMPTY_BOOLEAN_ARRAY();
-    return NS_OK;
-  }
-
-  // We only check the first element's type. If the array has mixed types,
-  // we'll throw an error during actual conversion.
-  JS::RootedValue element(aCx);
-  NS_ENSURE_TRUE(CheckJS(aCx, JS_GetElement(aCx, aData, 0, &element)),
-                 NS_ERROR_FAILURE);
-
-  if (element.isBoolean()) {
-    return BoxArrayPrimitive<bool, &JS::Value::isBoolean, &JS::Value::toBoolean,
-                             jni::BooleanArray, &jni::BooleanArray::New>(
-        aCx, aData, aOut, length, element);
-  }
-
-  if (element.isInt32()) {
-    nsresult rv =
-        BoxArrayPrimitive<int32_t, &JS::Value::isInt32, &JS::Value::toInt32,
-                          jni::IntArray, &jni::IntArray::New>(aCx, aData, aOut,
-                                                              length, element);
-    if (rv != NS_ERROR_INVALID_ARG) {
-      return rv;
-    }
-    // Not int32, but we can still try a double array.
-  }
-
-  if (element.isNumber()) {
-    return BoxArrayPrimitive<double, &JS::Value::isNumber, &JS::Value::toNumber,
-                             jni::DoubleArray, &jni::DoubleArray::New>(
-        aCx, aData, aOut, length, element);
-  }
-
-  if (element.isNullOrUndefined() || element.isString()) {
-    const auto isString = [](JS::HandleValue val) -> bool {
-      return val.isString();
-    };
-    nsresult rv = BoxArrayObject<jni::String, &BoxString>(
-        aCx, aData, aOut, length, element, isString);
-    if (element.isString() || rv != NS_ERROR_INVALID_ARG) {
-      return rv;
-    }
-    // First element was null/undefined, so it may still be an object array.
-  }
-
-  const auto isObject = [aCx](JS::HandleValue val) -> bool {
-    if (!val.isObject()) {
-      return false;
-    }
-    bool array = false;
-    JS::RootedObject obj(aCx, &val.toObject());
-    // We don't support array of arrays.
-    return CheckJS(aCx, JS::IsArrayObject(aCx, obj, &array)) && !array;
-  };
-
-  if (element.isNullOrUndefined() || isObject(element)) {
-    return BoxArrayObject<java::GeckoBundle, &BoxObject>(
-        aCx, aData, aOut, length, element, isObject);
-  }
-
-  NS_WARNING("Unknown type");
-  return NS_ERROR_INVALID_ARG;
-}
-
-nsresult BoxValue(JSContext* aCx, JS::HandleValue aData,
-                  jni::Object::LocalRef& aOut);
-
-nsresult BoxObject(JSContext* aCx, JS::HandleValue aData,
-                   jni::Object::LocalRef& aOut) {
-  if (aData.isNullOrUndefined()) {
-    aOut = nullptr;
-    return NS_OK;
-  }
-
-  MOZ_ASSERT(aData.isObject());
-
-  JS::Rooted<JS::IdVector> ids(aCx, JS::IdVector(aCx));
-  JS::RootedObject obj(aCx, &aData.toObject());
-
-  bool isArray = false;
-  if (CheckJS(aCx, JS::IsArrayObject(aCx, obj, &isArray)) && isArray) {
-    return BoxArray(aCx, obj, aOut);
-  }
-
-  NS_ENSURE_TRUE(CheckJS(aCx, JS_Enumerate(aCx, obj, &ids)), NS_ERROR_FAILURE);
-
-  const size_t length = ids.length();
-  auto keys = jni::ObjectArray::New<jni::String>(length);
-  auto values = jni::ObjectArray::New<jni::Object>(length);
-
-  // Iterate through each property of the JS object.
-  for (size_t i = 0; i < ids.length(); i++) {
-    const JS::RootedId id(aCx, ids[i]);
-    JS::RootedValue idVal(aCx);
-    JS::RootedValue val(aCx);
-    jni::Object::LocalRef key(aOut.Env());
-    jni::Object::LocalRef value(aOut.Env());
-
-    NS_ENSURE_TRUE(CheckJS(aCx, JS_IdToValue(aCx, id, &idVal)),
-                   NS_ERROR_FAILURE);
-
-    JS::RootedString idStr(aCx, JS::ToString(aCx, idVal));
-    NS_ENSURE_TRUE(CheckJS(aCx, !!idStr), NS_ERROR_FAILURE);
-
-    idVal.setString(idStr);
-    NS_ENSURE_SUCCESS(BoxString(aCx, idVal, key), NS_ERROR_FAILURE);
-    NS_ENSURE_TRUE(CheckJS(aCx, JS_GetPropertyById(aCx, obj, id, &val)),
-                   NS_ERROR_FAILURE);
-
-    nsresult rv = BoxValue(aCx, val, value);
-    if (rv == NS_ERROR_INVALID_ARG && !JS_IsExceptionPending(aCx)) {
-      nsAutoJSString autoStr;
-      if (CheckJS(aCx, autoStr.init(aCx, idVal.toString()))) {
-        JS_ReportErrorUTF8(aCx, u8"Invalid event data property %s",
-                           NS_ConvertUTF16toUTF8(autoStr).get());
-      }
-    }
-    NS_ENSURE_SUCCESS(rv, rv);
-
-    keys->SetElement(i, key);
-    values->SetElement(i, value);
-  }
-
-  aOut = java::GeckoBundle::New(keys, values);
-  return NS_OK;
-}
-
-nsresult BoxValue(JSContext* aCx, JS::HandleValue aData,
-                  jni::Object::LocalRef& aOut) {
-  if (aData.isNullOrUndefined()) {
-    aOut = nullptr;
-  } else if (aData.isBoolean()) {
-    aOut = aData.toBoolean() ? java::sdk::Boolean::TRUE()
-                             : java::sdk::Boolean::FALSE();
-  } else if (aData.isInt32()) {
-    aOut = java::sdk::Integer::ValueOf(aData.toInt32());
-  } else if (aData.isNumber()) {
-    aOut = java::sdk::Double::New(aData.toNumber());
-  } else if (aData.isString()) {
-    return BoxString(aCx, aData, aOut);
-  } else if (aData.isObject()) {
-    return BoxObject(aCx, aData, aOut);
-  } else {
-    NS_WARNING("Unknown type");
-    return NS_ERROR_INVALID_ARG;
-  }
-  return NS_OK;
-}
-
-nsresult BoxData(const nsAString& aEvent, JSContext* aCx, JS::HandleValue aData,
-                 jni::Object::LocalRef& aOut, bool aObjectOnly) {
-  nsresult rv = NS_ERROR_INVALID_ARG;
-
-  if (!aObjectOnly) {
-    rv = BoxValue(aCx, aData, aOut);
-  } else if (aData.isObject() || aData.isNullOrUndefined()) {
-    rv = BoxObject(aCx, aData, aOut);
-  }
+nsresult BoxData(const nsAString& aEvent, JSContext* aCx,
+                 JS::Handle<JS::Value> aData, jni::Object::LocalRef& aOut,
+                 bool aObjectOnly) {
+  nsresult rv = jni::BoxData(aCx, aData, aOut, aObjectOnly);
   if (rv != NS_ERROR_INVALID_ARG) {
     return rv;
   }
@@ -312,7 +57,7 @@ nsresult BoxData(const nsAString& aEvent, JSContext* aCx, JS::HandleValue aData,
 }
 
 nsresult UnboxString(JSContext* aCx, const jni::Object::LocalRef& aData,
-                     JS::MutableHandleValue aOut) {
+                     JS::MutableHandle<JS::Value> aOut) {
   if (!aData) {
     aOut.setNull();
     return NS_OK;
@@ -335,7 +80,7 @@ nsresult UnboxString(JSContext* aCx, const jni::Object::LocalRef& aData,
     env->ExceptionClear();
   });
 
-  JS::RootedString str(
+  JS::Rooted<JSString*> str(
       aCx,
       JS_NewUCStringCopyN(aCx, reinterpret_cast<const char16_t*>(jchars), len));
   NS_ENSURE_TRUE(CheckJS(aCx, !!str), NS_ERROR_FAILURE);
@@ -345,10 +90,10 @@ nsresult UnboxString(JSContext* aCx, const jni::Object::LocalRef& aData,
 }
 
 nsresult UnboxValue(JSContext* aCx, const jni::Object::LocalRef& aData,
-                    JS::MutableHandleValue aOut);
+                    JS::MutableHandle<JS::Value> aOut);
 
 nsresult UnboxBundle(JSContext* aCx, const jni::Object::LocalRef& aData,
-                     JS::MutableHandleValue aOut) {
+                     JS::MutableHandle<JS::Value> aOut) {
   if (!aData) {
     aOut.setNull();
     return NS_OK;
@@ -361,7 +106,7 @@ nsresult UnboxBundle(JSContext* aCx, const jni::Object::LocalRef& aData,
   jni::ObjectArray::LocalRef keys = bundle->Keys();
   jni::ObjectArray::LocalRef values = bundle->Values();
   const size_t len = keys->Length();
-  JS::RootedObject obj(aCx, JS_NewPlainObject(aCx));
+  JS::Rooted<JSObject*> obj(aCx, JS_NewPlainObject(aCx));
 
   NS_ENSURE_TRUE(CheckJS(aCx, !!obj), NS_ERROR_FAILURE);
   NS_ENSURE_TRUE(values->Length() == len, NS_ERROR_FAILURE);
@@ -380,7 +125,7 @@ nsresult UnboxBundle(JSContext* aCx, const jni::Object::LocalRef& aData,
       env->ExceptionClear();
     });
 
-    JS::RootedValue value(aCx);
+    JS::Rooted<JS::Value> value(aCx);
     nsresult rv = UnboxValue(aCx, values->GetElement(i), &value);
     if (rv == NS_ERROR_INVALID_ARG && !JS_IsExceptionPending(aCx)) {
       JS_ReportErrorUTF8(
@@ -407,7 +152,7 @@ template <typename Type, typename JNIType, typename ArrayType,
           void (JNIEnv::*ReleaseElements)(ArrayType, JNIType*, jint),
           JS::Value (*ToValue)(Type)>
 nsresult UnboxArrayPrimitive(JSContext* aCx, const jni::Object::LocalRef& aData,
-                             JS::MutableHandleValue aOut) {
+                             JS::MutableHandle<JS::Value> aOut) {
   JNIEnv* const env = aData.Env();
   const ArrayType jarray = ArrayType(aData.Get());
   JNIType* const array = (env->*GetElements)(jarray, nullptr);
@@ -431,8 +176,8 @@ nsresult UnboxArrayPrimitive(JSContext* aCx, const jni::Object::LocalRef& aData,
                    NS_ERROR_FAILURE);
   }
 
-  JS::RootedObject obj(aCx,
-                       JS::NewArrayObject(aCx, JS::HandleValueArray(elements)));
+  JS::Rooted<JSObject*> obj(
+      aCx, JS::NewArrayObject(aCx, JS::HandleValueArray(elements)));
   NS_ENSURE_TRUE(CheckJS(aCx, !!obj), NS_ERROR_FAILURE);
 
   aOut.setObject(*obj);
@@ -451,18 +196,18 @@ const char StringArray::name[] = "[Ljava/lang/String;";
 const char GeckoBundleArray::name[] = "[Lorg/mozilla/gecko/util/GeckoBundle;";
 
 template <nsresult (*Unbox)(JSContext*, const jni::Object::LocalRef&,
-                            JS::MutableHandleValue)>
+                            JS::MutableHandle<JS::Value>)>
 nsresult UnboxArrayObject(JSContext* aCx, const jni::Object::LocalRef& aData,
-                          JS::MutableHandleValue aOut) {
+                          JS::MutableHandle<JS::Value> aOut) {
   jni::ObjectArray::LocalRef array(aData.Env(),
                                    jni::ObjectArray::Ref::From(aData));
   const size_t len = array->Length();
-  JS::RootedObject obj(aCx, JS::NewArrayObject(aCx, len));
+  JS::Rooted<JSObject*> obj(aCx, JS::NewArrayObject(aCx, len));
   NS_ENSURE_TRUE(CheckJS(aCx, !!obj), NS_ERROR_FAILURE);
 
   for (size_t i = 0; i < len; i++) {
     jni::Object::LocalRef element = array->GetElement(i);
-    JS::RootedValue value(aCx);
+    JS::Rooted<JS::Value> value(aCx);
     nsresult rv = (*Unbox)(aCx, element, &value);
     NS_ENSURE_SUCCESS(rv, rv);
 
@@ -475,7 +220,7 @@ nsresult UnboxArrayObject(JSContext* aCx, const jni::Object::LocalRef& aData,
 }
 
 nsresult UnboxValue(JSContext* aCx, const jni::Object::LocalRef& aData,
-                    JS::MutableHandleValue aOut) {
+                    JS::MutableHandle<JS::Value> aOut) {
   using jni::Java2Native;
 
   if (!aData) {
@@ -528,7 +273,7 @@ nsresult UnboxValue(JSContext* aCx, const jni::Object::LocalRef& aData,
 }
 
 nsresult UnboxData(jni::String::Param aEvent, JSContext* aCx,
-                   jni::Object::Param aData, JS::MutableHandleValue aOut,
+                   jni::Object::Param aData, JS::MutableHandle<JS::Value> aOut,
                    bool aBundleOnly) {
   MOZ_ASSERT(NS_IsMainThread());
 
@@ -558,7 +303,7 @@ class JavaCallbackDelegate final : public nsIAndroidEventCallback {
 
   virtual ~JavaCallbackDelegate() {}
 
-  NS_IMETHOD Call(JSContext* aCx, JS::HandleValue aData,
+  NS_IMETHOD Call(JSContext* aCx, JS::Handle<JS::Value> aData,
                   void (java::EventCallback::*aCall)(jni::Object::Param)
                       const) {
     MOZ_ASSERT(NS_IsMainThread());
@@ -580,11 +325,11 @@ class JavaCallbackDelegate final : public nsIAndroidEventCallback {
 
   NS_DECL_ISUPPORTS
 
-  NS_IMETHOD OnSuccess(JS::HandleValue aData, JSContext* aCx) override {
+  NS_IMETHOD OnSuccess(JS::Handle<JS::Value> aData, JSContext* aCx) override {
     return Call(aCx, aData, &java::EventCallback::SendSuccess);
   }
 
-  NS_IMETHOD OnError(JS::HandleValue aData, JSContext* aCx) override {
+  NS_IMETHOD OnError(JS::Handle<JS::Value> aData, JSContext* aCx) override {
     return Call(aCx, aData, &java::EventCallback::SendError);
   }
 };
@@ -602,7 +347,7 @@ class NativeCallbackDelegateSupport final
   const nsCOMPtr<nsIGlobalObject> mGlobalObject;
 
   void Call(jni::Object::Param aData,
-            nsresult (nsIAndroidEventCallback::*aCall)(JS::HandleValue,
+            nsresult (nsIAndroidEventCallback::*aCall)(JS::Handle<JS::Value>,
                                                        JSContext*)) {
     MOZ_ASSERT(NS_IsMainThread());
 
@@ -611,7 +356,7 @@ class NativeCallbackDelegateSupport final
     dom::AutoJSAPI jsapi;
     NS_ENSURE_TRUE_VOID(jsapi.Init(mGlobalObject));
 
-    JS::RootedValue data(jsapi.cx());
+    JS::Rooted<JS::Value> data(jsapi.cx());
     nsresult rv = UnboxData(u"callback"_ns, jsapi.cx(), aData, &data,
                             /* BundleOnly */ false);
     NS_ENSURE_SUCCESS_VOID(rv);
@@ -695,7 +440,7 @@ nsIGlobalObject* EventDispatcher::GetGlobalObject() {
 
 nsresult EventDispatcher::DispatchOnGecko(ListenersList* list,
                                           const nsAString& aEvent,
-                                          JS::HandleValue aData,
+                                          JS::Handle<JS::Value> aData,
                                           nsIAndroidEventCallback* aCallback) {
   MOZ_ASSERT(NS_IsMainThread());
   dom::AutoNoJSAPI nojsapi;
@@ -756,7 +501,8 @@ bool EventDispatcher::HasListener(const char16_t* aEvent) {
 }
 
 NS_IMETHODIMP
-EventDispatcher::Dispatch(JS::HandleValue aEvent, JS::HandleValue aData,
+EventDispatcher::Dispatch(JS::Handle<JS::Value> aEvent,
+                          JS::Handle<JS::Value> aData,
                           nsIAndroidEventCallback* aCallback,
                           nsIAndroidEventFinalizer* aFinalizer,
                           JSContext* aCx) {
@@ -812,7 +558,7 @@ nsresult EventDispatcher::Dispatch(const char16_t* aEvent,
   if (list) {
     dom::AutoJSAPI jsapi;
     NS_ENSURE_TRUE(jsapi.Init(GetGlobalObject()), NS_ERROR_FAILURE);
-    JS::RootedValue data(jsapi.cx());
+    JS::Rooted<JS::Value> data(jsapi.cx());
     nsresult rv = UnboxData(/* Event */ nullptr, jsapi.cx(), aData, &data,
                             /* BundleOnly */ true);
     NS_ENSURE_SUCCESS(rv, rv);
@@ -828,7 +574,8 @@ nsresult EventDispatcher::Dispatch(const char16_t* aEvent,
   return NS_OK;
 }
 
-nsresult EventDispatcher::IterateEvents(JSContext* aCx, JS::HandleValue aEvents,
+nsresult EventDispatcher::IterateEvents(JSContext* aCx,
+                                        JS::Handle<JS::Value> aEvents,
                                         IterateEventsCallback aCallback,
                                         nsIAndroidEventListener* aListener) {
   MOZ_ASSERT(NS_IsMainThread());
@@ -836,7 +583,7 @@ nsresult EventDispatcher::IterateEvents(JSContext* aCx, JS::HandleValue aEvents,
   MutexAutoLock lock(mLock);
 
   auto processEvent = [this, aCx, aCallback,
-                       aListener](JS::HandleValue event) -> nsresult {
+                       aListener](JS::Handle<JS::Value> event) -> nsresult {
     nsAutoJSString str;
     NS_ENSURE_TRUE(CheckJS(aCx, str.init(aCx, event.toString())),
                    NS_ERROR_OUT_OF_MEMORY);
@@ -853,14 +600,14 @@ nsresult EventDispatcher::IterateEvents(JSContext* aCx, JS::HandleValue aEvents,
                  NS_ERROR_INVALID_ARG);
   NS_ENSURE_TRUE(isArray, NS_ERROR_INVALID_ARG);
 
-  JS::RootedObject events(aCx, &aEvents.toObject());
+  JS::Rooted<JSObject*> events(aCx, &aEvents.toObject());
   uint32_t length = 0;
   NS_ENSURE_TRUE(CheckJS(aCx, JS::GetArrayLength(aCx, events, &length)),
                  NS_ERROR_INVALID_ARG);
   NS_ENSURE_TRUE(length, NS_ERROR_INVALID_ARG);
 
   for (size_t i = 0; i < length; i++) {
-    JS::RootedValue event(aCx);
+    JS::Rooted<JS::Value> event(aCx);
     NS_ENSURE_TRUE(CheckJS(aCx, JS_GetElement(aCx, events, i, &event)),
                    NS_ERROR_INVALID_ARG);
     NS_ENSURE_TRUE(event.isString(), NS_ERROR_INVALID_ARG);
@@ -888,7 +635,8 @@ nsresult EventDispatcher::RegisterEventLocked(
 
 NS_IMETHODIMP
 EventDispatcher::RegisterListener(nsIAndroidEventListener* aListener,
-                                  JS::HandleValue aEvents, JSContext* aCx) {
+                                  JS::Handle<JS::Value> aEvents,
+                                  JSContext* aCx) {
   return IterateEvents(aCx, aEvents, &EventDispatcher::RegisterEventLocked,
                        aListener);
 }
@@ -925,7 +673,8 @@ nsresult EventDispatcher::UnregisterEventLocked(
 
 NS_IMETHODIMP
 EventDispatcher::UnregisterListener(nsIAndroidEventListener* aListener,
-                                    JS::HandleValue aEvents, JSContext* aCx) {
+                                    JS::Handle<JS::Value> aEvents,
+                                    JSContext* aCx) {
   return IterateEvents(aCx, aEvents, &EventDispatcher::UnregisterEventLocked,
                        aListener);
 }
@@ -999,7 +748,7 @@ void EventDispatcher::DispatchToGecko(jni::String::Param aEvent,
   dom::AutoJSAPI jsapi;
   NS_ENSURE_TRUE_VOID(jsapi.Init(GetGlobalObject()));
 
-  JS::RootedValue data(jsapi.cx());
+  JS::Rooted<JS::Value> data(jsapi.cx());
   nsresult rv = UnboxData(aEvent, jsapi.cx(), aData, &data,
                           /* BundleOnly */ true);
   NS_ENSURE_SUCCESS_VOID(rv);
@@ -1015,7 +764,7 @@ void EventDispatcher::DispatchToGecko(jni::String::Param aEvent,
 
 /* static */
 nsresult EventDispatcher::UnboxBundle(JSContext* aCx, jni::Object::Param aData,
-                                      JS::MutableHandleValue aOut) {
+                                      JS::MutableHandle<JS::Value> aOut) {
   return detail::UnboxBundle(aCx, aData, aOut);
 }
 
