@@ -15,6 +15,8 @@
 #include "mozilla/java/ImageWrappers.h"
 #include "nsNetUtil.h"
 
+using namespace mozilla::gfx;
+
 namespace mozilla {
 namespace widget {
 
@@ -44,7 +46,7 @@ class ImageCallbackHelper : public imgIContainerCallback,
         reinterpret_cast<int8_t*>(aSourceSurface.GetData()),
         aSourceSurface.GetStride() * height);
     auto bitmap = java::sdk::Bitmap::CreateBitmap(
-        width, height, java::sdk::Config::ARGB_8888());
+        width, height, java::sdk::Bitmap::Config::ARGB_8888());
     bitmap->CopyPixelsFromBuffer(pixels);
     mResult->Complete(bitmap);
     gDecodeRequests.remove(this);
