@@ -825,6 +825,43 @@ class ArrayRefBase : public ObjectBase<TypedObject<JNIType>, JNIType> {
     return Base::LocalRef::Adopt(jenv, result);
   }
 
+  static typename Base::LocalRef New(const ElementType* data, size_t length,
+                                     const fallible_t&) {
+    using JNIElemType = typename detail::TypeAdapter<ElementType>::JNIType;
+    static_assert(sizeof(ElementType) == sizeof(JNIElemType),
+                  "Size of native type must match size of JNI type");
+    JNIEnv* const jenv = mozilla::jni::GetEnvForThread();
+    auto result = (jenv->*detail::TypeAdapter<ElementType>::NewArray)(length);
+    if (jenv->ExceptionCheck()) {
+      if (!IsOOMException(jenv)) {
+        // This exception isn't excepted due not to OOM. This is unrecoverable
+        // error.
+        MOZ_CATCH_JNI_EXCEPTION(jenv);
+      }
+#ifdef MOZ_CHECK_JNI
+      jenv->ExceptionDescribe();
+#endif
+      jenv->ExceptionClear();
+      return Base::LocalRef::Adopt(jenv, nullptr);
+    }
+    (jenv->*detail::TypeAdapter<ElementType>::SetArray)(
+        result, jsize(0), length, reinterpret_cast<const JNIElemType*>(data));
+    if (jenv->ExceptionCheck()) {
+      if (!IsOOMException(jenv)) {
+        // This exception isn't excepted due not to OOM. This is unrecoverable
+        // error.
+        MOZ_CATCH_JNI_EXCEPTION(jenv);
+      }
+#ifdef MOZ_CHECK_JNI
+      jenv->ExceptionDescribe();
+#endif
+      jenv->ExceptionClear();
+      jenv->DeleteLocalRef(result);
+      return Base::LocalRef::Adopt(jenv, nullptr);
+    }
+    return Base::LocalRef::Adopt(jenv, result);
+  }
+
   size_t Length() const {
     const size_t ret = Base::Env()->GetArrayLength(Base::Instance());
     MOZ_CATCH_JNI_EXCEPTION(Base::Env());
