@@ -44,10 +44,10 @@ class GLPresenter;
 }  // namespace
 
 namespace mozilla {
+enum class NativeKeyBindingsType : uint8_t;
+
 class InputData;
 class PanGestureInput;
-class SwipeTracker;
-struct SwipeEventQueue;
 class VibrancyManager;
 namespace layers {
 class GLManager;
@@ -291,7 +291,7 @@ class nsChildView final : public nsBaseWidget {
   // nsIWidget interface
   [[nodiscard]] virtual nsresult Create(nsIWidget* aParent, nsNativeWidget aNativeParent,
                                         const LayoutDeviceIntRect& aRect,
-                                        nsWidgetInitData* aInitData = nullptr) override;
+                                        InitData* = nullptr) override;
 
   virtual void Destroy() override;
 
@@ -308,6 +308,10 @@ class nsChildView final : public nsBaseWidget {
 
   virtual void Enable(bool aState) override;
   virtual bool IsEnabled() const override;
+
+  virtual nsSizeMode SizeMode() override { return mSizeMode; }
+  virtual void SetSizeMode(nsSizeMode aMode) override { mSizeMode = aMode; }
+
   virtual void SetFocus(Raise, mozilla::dom::CallerType aCallerType) override;
   virtual LayoutDeviceIntRect GetBounds() override;
   virtual LayoutDeviceIntRect GetClientBounds() override;
@@ -344,13 +348,9 @@ class nsChildView final : public nsBaseWidget {
   void EnsureContentLayerForMainThreadPainting();
 
   virtual void* GetNativeData(uint32_t aDataType) override;
-  virtual nsresult ConfigureChildren(const nsTArray<Configuration>& aConfigurations) override;
   virtual LayoutDeviceIntPoint WidgetToScreenOffset() override;
   virtual bool ShowsResizeIndicator(LayoutDeviceIntRect* aResizerRect) override { return false; }
 
-  static bool ConvertStatus(nsEventStatus aStatus) {
-    return aStatus == nsEventStatus_eConsumeNoDefault;
-  }
   virtual nsresult DispatchEvent(mozilla::WidgetGUIEvent* aEvent, nsEventStatus& aStatus) override;
 
   virtual bool WidgetTypeSupportsAcceleration() override;
@@ -377,7 +377,7 @@ class nsChildView final : public nsBaseWidget {
   [[nodiscard]] virtual nsresult AttachNativeKeyEvent(
       mozilla::WidgetKeyboardEvent& aEvent) override;
   MOZ_CAN_RUN_SCRIPT virtual bool GetEditCommands(
-      NativeKeyBindingsType aType, const mozilla::WidgetKeyboardEvent& aEvent,
+      mozilla::NativeKeyBindingsType aType, const mozilla::WidgetKeyboardEvent& aEvent,
       nsTArray<mozilla::CommandInt>& aCommands) override;
 
   virtual void SuppressAnimation(bool aSuppress) override;
@@ -414,9 +414,6 @@ class nsChildView final : public nsBaseWidget {
                                                      uint32_t aModifierFlags) override;
 
   // Mac specific methods
-
-  virtual bool DispatchWindowEvent(mozilla::WidgetGUIEvent& event);
-
   void WillPaintWindow();
   bool PaintWindow(LayoutDeviceIntRegion aRegion);
   bool PaintWindowInDrawTarget(mozilla::gfx::DrawTarget* aDT, const LayoutDeviceIntRegion& aRegion,
@@ -442,8 +439,6 @@ class nsChildView final : public nsBaseWidget {
   virtual void UpdateWindowDraggingRegion(const LayoutDeviceIntRegion& aRegion) override;
   LayoutDeviceIntRegion GetNonDraggableRegion() { return mNonDraggableRegion.Region(); }
 
-  virtual void ReportSwipeStarted(uint64_t aInputBlockId, bool aStartSwipe) override;
-
   virtual void LookUpDictionary(const nsAString& aText,
                                 const nsTArray<mozilla::FontRange>& aFontRangeArray,
                                 const bool aIsVertical,
@@ -485,15 +480,13 @@ class nsChildView final : public nsBaseWidget {
 
   virtual LayoutDeviceIntPoint GetClientOffset() override;
 
-  void DispatchAPZWheelInputEvent(mozilla::InputData& aEvent, bool aCanTriggerSwipe);
+  void DispatchAPZWheelInputEvent(mozilla::InputData& aEvent);
   nsEventStatus DispatchAPZInputEvent(mozilla::InputData& aEvent);
 
   void DispatchDoubleTapGesture(mozilla::TimeStamp aEventTimeStamp,
                                 LayoutDeviceIntPoint aScreenPosition,
                                 mozilla::Modifiers aModifiers);
 
-  void SwipeFinished();
-
   // Called when the main thread enters a phase during which visual changes
   // are imminent and any layer updates on the compositor thread would interfere
   // with visual atomicity.
@@ -511,6 +504,15 @@ class nsChildView final : public nsBaseWidget {
   // compositor thread again.
   void UnsuspendAsyncCATransactions();
 
+  // Called by nsCocoaWindow when the window's fullscreen state changes.
+  void UpdateFullscreen(bool aFullscreen);
+
+#ifdef DEBUG
+  // test only.
+  virtual nsresult SetHiDPIMode(bool aHiDPI) override;
+  virtual nsresult RestoreHiDPIMode() override;
+#endif
+
  protected:
   virtual ~nsChildView();
 
@@ -531,15 +533,6 @@ class nsChildView final : public nsBaseWidget {
 
   nsIWidget* GetWidgetForListenerEvents();
 
-  struct SwipeInfo {
-    bool wantsSwipe;
-    uint32_t allowedDirections;
-  };
-
-  SwipeInfo SendMayStartSwipe(const mozilla::PanGestureInput& aSwipeStartEvent);
-  void TrackScrollEventAsSwipe(const mozilla::PanGestureInput& aSwipeStartEvent,
-                               uint32_t aAllowedDirections);
-
  protected:
   ChildView* mView;  // my parallel cocoa view, [STRONG]
   RefPtr<mozilla::widget::TextInputHandler> mTextInputHandler;
@@ -557,7 +550,7 @@ class nsChildView final : public nsBaseWidget {
   // Held while the compositor (or WR renderer) thread is compositing.
   // Protects from tearing down the view during compositing and from presenting
   // half-composited layers to the screen.
-  mozilla::Mutex mCompositingLock;
+  mozilla::Mutex mCompositingLock MOZ_UNANNOTATED;
 
   mozilla::ViewRegion mNonDraggableRegion;
 
@@ -568,6 +561,7 @@ class nsChildView final : public nsBaseWidget {
   mutable CGFloat mBackingScaleFactor;
 
   bool mVisible;
+  nsSizeMode mSizeMode;
   bool mDrawing;
   bool mIsDispatchPaint;  // Is a paint event being dispatched
 
@@ -584,19 +578,9 @@ class nsChildView final : public nsBaseWidget {
   LayoutDeviceIntRegion mContentLayerInvalidRegion;
 
   mozilla::UniquePtr<mozilla::VibrancyManager> mVibrancyManager;
-  RefPtr<mozilla::SwipeTracker> mSwipeTracker;
-  mozilla::UniquePtr<mozilla::SwipeEventQueue> mSwipeEventQueue;
 
   RefPtr<mozilla::CancelableRunnable> mUnsuspendAsyncCATransactionsRunnable;
 
-  // This flag is only used when APZ is off. It indicates that the current pan
-  // gesture was processed as a swipe. Sometimes the swipe animation can finish
-  // before momentum events of the pan gesture have stopped firing, so this
-  // flag tells us that we shouldn't allow the remaining events to cause
-  // scrolling. It is reset to false once a new gesture starts (as indicated by
-  // a PANGESTURE_(MAY)START event).
-  bool mCurrentPanGestureBelongsToSwipe;
-
   static uint32_t sLastInputEventCount;
 
   // This is used by SynthesizeNativeTouchPoint to maintain state between
