@@ -116,7 +116,7 @@ class nsMenuX final : public nsMenuParentX,
   // Fires the popupshowing event and returns whether the handler allows the popup to open.
   // When calling this method, the caller must hold a strong reference to this object, because other
   // references to this object can be dropped during the handling of the DOM event.
-  bool OnOpen();
+  MOZ_CAN_RUN_SCRIPT bool OnOpen();
 
   void PopupShowingEventWasSentAndApprovedExternally() { DidFirePopupShowing(); }
 
@@ -124,16 +124,14 @@ class nsMenuX final : public nsMenuParentX,
   // Ignored if the menu is already considered open.
   // When calling this method, the caller must hold a strong reference to this object, because other
   // references to this object can be dropped during the handling of the DOM event.
-  void MenuOpened();
+  // TODO: Convert this to MOZ_CAN_RUN_SCRIPT (bug 1415230)
+  MOZ_CAN_RUN_SCRIPT_BOUNDARY void MenuOpened();
 
   // Called from the menu delegate during menuDidClose, or to simulate closing.
   // Ignored if the menu is already considered closed.
   // When calling this method, the caller must hold a strong reference to this object, because other
   // references to this object can be dropped during the handling of the DOM event.
-  // If aEntireMenuClosingDueToActivateItem is true, it means that popuphiding/popuphidden events
-  // can be delayed until the event loop for the menu is exited. If this is a submenu, this is
-  // usually not possible because the rest of the menu might stay open.
-  void MenuClosed(bool aEntireMenuClosingDueToActivateItem = false);
+  void MenuClosed();
 
   // Close the menu if it's open, and flush any pending popuphiding / popuphidden events.
   bool Close();
@@ -163,6 +161,7 @@ class nsMenuX final : public nsMenuParentX,
   void Dump(uint32_t aIndent) const;
 
   static bool IsXULHelpMenu(nsIContent* aMenuContent);
+  static bool IsXULWindowMenu(nsIContent* aMenuContent);
 
   // Set an observer that gets notified of menu opening and closing.
   // The menu does not keep a strong reference the observer. The observer must
@@ -203,22 +202,24 @@ class nsMenuX final : public nsMenuParentX,
   NSInteger CalculateNativeInsertionPoint(const MenuChild& aChild);
 
   // Fires the popupshown event.
-  void MenuOpenedAsync();
+  MOZ_CAN_RUN_SCRIPT void MenuOpenedAsync();
 
   // Called from mPendingAsyncMenuCloseRunnable asynchronously after MenuClosed(), so that it runs
   // after any potential menuItemHit calls for clicked menu items.
   // Fires popuphiding and popuphidden events.
   // When calling this method, the caller must hold a strong reference to this object, because other
   // references to this object can be dropped during the handling of the DOM event.
-  void MenuClosedAsync();
+  MOZ_CAN_RUN_SCRIPT void MenuClosedAsync();
 
   // If mPendingAsyncMenuOpenRunnable is non-null, call MenuOpenedAsync() to send out the pending
   // popupshown event.
-  void FlushMenuOpenedRunnable();
+  // TODO: Convert this to MOZ_CAN_RUN_SCRIPT (bug 1415230)
+  MOZ_CAN_RUN_SCRIPT_BOUNDARY void FlushMenuOpenedRunnable();
 
   // If mPendingAsyncMenuCloseRunnable is non-null, call MenuClosedAsync() to send out pending
   // popuphiding/popuphidden events.
-  void FlushMenuClosedRunnable();
+  // TODO: Convert this to MOZ_CAN_RUN_SCRIPT (bug 1415230)
+  MOZ_CAN_RUN_SCRIPT_BOUNDARY void FlushMenuClosedRunnable();
 
   // Make sure the NSMenu contains at least one item, even if mVisibleItemsCount is zero.
   // Otherwise it won't open.
@@ -249,10 +250,15 @@ class nsMenuX final : public nsMenuParentX,
   // menuItemHit.
   RefPtr<mozilla::CancelableRunnable> mPendingAsyncMenuCloseRunnable;
 
-  // Any runnables for running asynchronous command events.
-  // These are only used during automated tests, via ActivateItemAfterClosing.
-  // We keep track of them here so that we can ensure they're run before popuphiding/popuphidden.
-  nsTArray<RefPtr<mozilla::Runnable>> mPendingCommandRunnables;
+  struct PendingCommandEvent {
+    RefPtr<nsMenuItemX> mMenuItem;
+    NSEventModifierFlags mModifiers;
+    int16_t mButton;
+  };
+
+  // Any pending command events.
+  // These are queued by ActivateItemAfterClosing and run by MenuClosedAsync.
+  nsTArray<PendingCommandEvent> mPendingCommandEvents;
 
   GeckoNSMenu* mNativeMenu = nil;     // [strong]
   MenuDelegate* mMenuDelegate = nil;  // [strong]
