@@ -27,6 +27,8 @@ class CompositorWidgetChild final : public PCompositorWidgetChild,
   mozilla::ipc::IPCResult RecvUnobserveVsync() override;
 
   void NotifyClientSizeChanged(const LayoutDeviceIntSize& aClientSize) override;
+  void DisableRendering() override;
+  void EnableRendering(const uintptr_t aXWindow, const bool aShaped) override;
 
  private:
   RefPtr<CompositorVsyncDispatcher> mVsyncDispatcher;
