@@ -26,7 +26,6 @@ class GfxInfo final : public GfxInfoBase {
   NS_IMETHOD GetHasBattery(bool* aHasBattery) override;
   NS_IMETHOD GetCleartypeParameters(nsAString& aCleartypeParams) override;
   NS_IMETHOD GetWindowProtocol(nsAString& aWindowProtocol) override;
-  NS_IMETHOD GetDesktopEnvironment(nsAString& aDesktopEnvironment) override;
   NS_IMETHOD GetTestType(nsAString& aTestType) override;
   NS_IMETHOD GetAdapterDescription(nsAString& aAdapterDescription) override;
   NS_IMETHOD GetAdapterDriver(nsAString& aAdapterDriver) override;
@@ -48,9 +47,6 @@ class GfxInfo final : public GfxInfoBase {
       nsAString& aAdapterDriverVersion) override;
   NS_IMETHOD GetAdapterDriverDate2(nsAString& aAdapterDriverDate) override;
   NS_IMETHOD GetIsGPU2Active(bool* aIsGPU2Active) override;
-  NS_IMETHOD GetDisplayInfo(nsTArray<nsString>& aDisplayInfo) override;
-  NS_IMETHOD GetDisplayWidth(nsTArray<uint32_t>& aDisplayWidth) override;
-  NS_IMETHOD GetDisplayHeight(nsTArray<uint32_t>& aDisplayHeight) override;
   NS_IMETHOD GetDrmRenderDevice(nsACString& aDrmRenderDevice) override;
   using GfxInfoBase::GetFeatureStatus;
   using GfxInfoBase::GetFeatureSuggestedDriverVersion;
@@ -59,6 +55,8 @@ class GfxInfo final : public GfxInfoBase {
 
   NS_IMETHOD_(void) GetData() override;
 
+  static bool FireGLXTestProcess();
+
 #ifdef DEBUG
   NS_DECL_ISUPPORTS_INHERITED
   NS_DECL_NSIGFXINFODEBUG
@@ -67,6 +65,9 @@ class GfxInfo final : public GfxInfoBase {
  protected:
   ~GfxInfo() = default;
 
+  OperatingSystem GetOperatingSystem() override {
+    return OperatingSystem::Linux;
+  }
   virtual nsresult GetFeatureStatusImpl(
       int32_t aFeature, int32_t* aStatus, nsAString& aSuggestedDriverVersion,
       const nsTArray<GfxDriverInfo>& aDriverInfo, nsACString& aFailureId,
@@ -79,8 +80,11 @@ class GfxInfo final : public GfxInfoBase {
 
   virtual bool DoesDriverVendorMatch(const nsAString& aBlocklistVendor,
                                      const nsAString& aDriverVendor) override;
+  static int FireTestProcess(const nsAString& aBinaryFile, int* aOutPipe,
+                             const char** aStringArgs);
 
  private:
+  bool mInitialized = false;
   nsCString mVendorId;
   nsCString mDeviceId;
   nsCString mDriverVendor;
@@ -89,7 +93,6 @@ class GfxInfo final : public GfxInfoBase {
   uint32_t mAdapterRAM;
   nsCString mOS;
   nsCString mOSRelease;
-  nsAutoCStringN<16> mDesktopEnvironment;
   nsCString mTestType;
 
   nsCString mSecondaryVendorId;
@@ -114,7 +117,13 @@ class GfxInfo final : public GfxInfoBase {
   bool mIsXWayland;
   bool mHasMultipleGPUs;
   bool mGlxTestError;
+  mozilla::Maybe<bool> mIsVAAPISupported;
+  int mVAAPISupportedCodecs = 0;
 
+  static int sGLXTestPipe;
+  static pid_t sGLXTestPID;
+
+  void GetDataVAAPI();
   void AddCrashReportAnnotations();
 };
 
