@@ -7,6 +7,7 @@
 #include "WindowSurfaceProvider.h"
 
 #include "gfxPlatformGtk.h"
+#include "GtkCompositorWidget.h"
 #include "mozilla/gfx/Logging.h"
 #include "mozilla/layers/LayersTypes.h"
 #include "nsWindow.h"
@@ -41,10 +42,6 @@ WindowSurfaceProvider::WindowSurfaceProvider()
     : mWindowSurface(nullptr),
       mMutex("WindowSurfaceProvider"),
       mWindowSurfaceValid(false)
-#ifdef MOZ_WAYLAND
-      ,
-      mWidget(nullptr)
-#endif
 #ifdef MOZ_X11
       ,
       mIsShaped(false),
@@ -60,6 +57,11 @@ void WindowSurfaceProvider::Initialize(RefPtr<nsWindow> aWidget) {
   mWindowSurfaceValid = false;
   mWidget = std::move(aWidget);
 }
+void WindowSurfaceProvider::Initialize(GtkCompositorWidget* aCompositorWidget) {
+  mWindowSurfaceValid = false;
+  mCompositorWidget = aCompositorWidget;
+  mWidget = static_cast<nsWindow*>(aCompositorWidget->RealWidget());
+}
 #endif
 #ifdef MOZ_X11
 void WindowSurfaceProvider::Initialize(Window aWindow, Visual* aVisual,
@@ -93,7 +95,7 @@ RefPtr<WindowSurface> WindowSurfaceProvider::CreateWindowSurface() {
     if (!mWidget) {
       return nullptr;
     }
-    return MakeRefPtr<WindowSurfaceWaylandMB>(mWidget);
+    return MakeRefPtr<WindowSurfaceWaylandMB>(mWidget, mCompositorWidget);
   }
 #endif
 #ifdef MOZ_X11
@@ -170,7 +172,7 @@ void WindowSurfaceProvider::EndRemoteDrawingInRegion(
   if (GdkIsWaylandDisplay()) {
     // We're called too early or we're unmapped.
     // Don't draw anything.
-    if (!mWidget) {
+    if (!mWidget || !mWidget->IsMapped()) {
       return;
     }
     if (moz_container_wayland_is_commiting_to_parent(
@@ -179,7 +181,10 @@ void WindowSurfaceProvider::EndRemoteDrawingInRegion(
       // in main thread to sync with Gtk access to it.
       NS_DispatchToMainThread(NS_NewRunnableFunction(
           "WindowSurfaceProvider::EndRemoteDrawingInRegion",
-          [RefPtr{mWidget}, this, aInvalidRegion]() {
+          [widget = RefPtr{mWidget}, this, aInvalidRegion]() {
+            if (!widget->IsMapped()) {
+              return;
+            }
             MutexAutoLock lock(mMutex);
             // Commit to mWindowSurface only when we have a valid one.
             if (mWindowSurface && mWindowSurfaceValid) {
