@@ -5,13 +5,13 @@
 
 #include "nsDeviceContextSpecG.h"
 
+#include "mozilla/gfx/PrintPromise.h"
 #include "mozilla/gfx/PrintTargetPDF.h"
 #include "mozilla/Logging.h"
 #include "mozilla/Services.h"
 #include "mozilla/GUniquePtr.h"
 #include "mozilla/WidgetUtilsGtk.h"
 
-#include "plstr.h"
 #include "prenv.h" /* for PR_GetEnv */
 
 #include "nsComponentManagerUtils.h"
@@ -46,6 +46,7 @@
 using namespace mozilla;
 
 using mozilla::gfx::IntSize;
+using mozilla::gfx::PrintEndDocumentPromise;
 using mozilla::gfx::PrintTarget;
 using mozilla::gfx::PrintTargetPDF;
 
@@ -335,7 +336,7 @@ nsDeviceContextSpecGTK::BeginDocument(const nsAString& aTitle,
   return NS_OK;
 }
 
-NS_IMETHODIMP nsDeviceContextSpecGTK::EndDocument() {
+RefPtr<PrintEndDocumentPromise> nsDeviceContextSpecGTK::EndDocument() {
   switch (mPrintSettings->GetOutputDestination()) {
     case nsIPrintSettings::kOutputDestinationPrinter: {
       // At this point, we might have a GtkPrinter set up in nsPrintSettingsGTK,
@@ -367,28 +368,35 @@ NS_IMETHODIMP nsDeviceContextSpecGTK::EndDocument() {
 
       nsresult rv =
           NS_NewLocalFile(targetPath, false, getter_AddRefs(destFile));
-      NS_ENSURE_SUCCESS(rv, rv);
+      if (NS_FAILED(rv)) {
+        return PrintEndDocumentPromise::CreateAndReject(rv, __func__);
+      }
 
-      nsAutoString destLeafName;
-      rv = destFile->GetLeafName(destLeafName);
-      NS_ENSURE_SUCCESS(rv, rv);
+      return nsIDeviceContextSpec::EndDocumentAsync(
+          __func__,
+          [destFile = std::move(destFile),
+           spoolFile = std::move(mSpoolFile)]() -> nsresult {
+            nsAutoString destLeafName;
+            auto rv = destFile->GetLeafName(destLeafName);
+            NS_ENSURE_SUCCESS(rv, rv);
 
-      nsCOMPtr<nsIFile> destDir;
-      rv = destFile->GetParent(getter_AddRefs(destDir));
-      NS_ENSURE_SUCCESS(rv, rv);
+            nsCOMPtr<nsIFile> destDir;
+            rv = destFile->GetParent(getter_AddRefs(destDir));
+            NS_ENSURE_SUCCESS(rv, rv);
 
-      rv = mSpoolFile->MoveTo(destDir, destLeafName);
-      NS_ENSURE_SUCCESS(rv, rv);
+            rv = spoolFile->MoveTo(destDir, destLeafName);
+            NS_ENSURE_SUCCESS(rv, rv);
 
-      mSpoolFile = nullptr;
+            // This is the standard way to get the UNIX umask. Ugh.
+            mode_t mask = umask(0);
+            umask(mask);
+            // If you're not familiar with umasks, they contain the bits of what
+            // NOT to set in the permissions (thats because files and
+            // directories have different numbers of bits for their permissions)
+            destFile->SetPermissions(0666 & ~(mask));
+            return NS_OK;
+          });
 
-      // This is the standard way to get the UNIX umask. Ugh.
-      mode_t mask = umask(0);
-      umask(mask);
-      // If you're not familiar with umasks, they contain the bits of what NOT
-      // to set in the permissions (thats because files and directories have
-      // different numbers of bits for their permissions)
-      destFile->SetPermissions(0666 & ~(mask));
       break;
     }
     case nsIPrintSettings::kOutputDestinationStream:
@@ -396,5 +404,5 @@ NS_IMETHODIMP nsDeviceContextSpecGTK::EndDocument() {
       MOZ_ASSERT(!mSpoolFile);
       break;
   }
-  return NS_OK;
+  return PrintEndDocumentPromise::CreateAndResolve(true, __func__);
 }
