@@ -9,12 +9,13 @@
 
 #include "mozilla/RefPtr.h"
 #include "nsBaseDragService.h"
+#include "nsCOMArray.h"
 #include "nsIObserver.h"
 #include <gtk/gtk.h>
+#include "nsITimer.h"
 
 class nsICookieJarSettings;
 class nsWindow;
-class nsWaylandDragContext;
 
 namespace mozilla {
 namespace gfx {
@@ -31,7 +32,6 @@ class nsDragService final : public nsBaseDragService, public nsIObserver {
   nsDragService();
 
   NS_DECL_ISUPPORTS_INHERITED
-
   NS_DECL_NSIOBSERVER
 
   // nsBaseDragService
@@ -58,6 +58,10 @@ class nsDragService final : public nsBaseDragService, public nsIObserver {
   NS_IMETHOD IsDataFlavorSupported(const char* aDataFlavor,
                                    bool* _retval) override;
 
+  // Update Drag&Drop state according child process state.
+  // UpdateDragEffect() is called by IPC bridge when child process
+  // accepts/denies D&D operation and uses stored
+  // mTargetDragContextForRemote context.
   NS_IMETHOD UpdateDragEffect() override;
 
   // Methods called from nsWindow to handle responding to GTK drag
@@ -70,12 +74,10 @@ class nsDragService final : public nsBaseDragService, public nsIObserver {
                           guint aInfo, guint32 aTime);
 
   gboolean ScheduleMotionEvent(nsWindow* aWindow, GdkDragContext* aDragContext,
-                               nsWaylandDragContext* aPendingWaylandDragContext,
                                mozilla::LayoutDeviceIntPoint aWindowPoint,
                                guint aTime);
   void ScheduleLeaveEvent();
   gboolean ScheduleDropEvent(nsWindow* aWindow, GdkDragContext* aDragContext,
-                             nsWaylandDragContext* aPendingWaylandDragContext,
                              mozilla::LayoutDeviceIntPoint aWindowPoint,
                              guint aTime);
 
@@ -91,12 +93,37 @@ class nsDragService final : public nsBaseDragService, public nsIObserver {
   void SourceEndDragSession(GdkDragContext* aContext, gint aResult);
   void SourceDataGet(GtkWidget* widget, GdkDragContext* context,
                      GtkSelectionData* selection_data, guint32 aTime);
+  bool SourceDataGetText(nsITransferable* aItem, const nsACString& aMIMEType,
+                         bool aNeedToDoConversionToPlainText,
+                         GtkSelectionData* aSelectionData);
+  void SourceDataGetImage(nsITransferable* aItem,
+                          GtkSelectionData* aSelectionData);
+  void SourceDataGetXDND(nsITransferable* aItem, GdkDragContext* aContext,
+                         GtkSelectionData* aSelectionData);
+  void SourceDataGetUriList(GdkDragContext* aContext,
+                            GtkSelectionData* aSelectionData,
+                            uint32_t aDragItems);
+  bool SourceDataAppendURLFileItem(nsACString& aURI, nsITransferable* aItem);
+  bool SourceDataAppendURLItem(nsITransferable* aItem, bool aExternalDrop,
+                               nsACString& aURI);
 
   void SourceBeginDrag(GdkDragContext* aContext);
 
   // set the drag icon during drag-begin
   void SetDragIcon(GdkDragContext* aContext);
 
+  class AutoEventLoop {
+    RefPtr<nsDragService> mService;
+
+   public:
+    explicit AutoEventLoop(RefPtr<nsDragService> aService)
+        : mService(std::move(aService)) {
+      mService->mEventLoopDepth++;
+    }
+    ~AutoEventLoop() { mService->mEventLoopDepth--; }
+  };
+  int GetLoopDepth() const { return mEventLoopDepth; };
+
  protected:
   virtual ~nsDragService();
 
@@ -116,6 +143,10 @@ class nsDragService final : public nsBaseDragService, public nsIObserver {
   // mTaskSource is the GSource id for the task that is either scheduled
   // or currently running.  It is 0 if no task is scheduled or running.
   guint mTaskSource;
+  bool mScheduledTaskIsRunning;
+
+  // Where the drag begins. We need to keep it open on Wayland.
+  RefPtr<nsWindow> mSourceWindow;
 
   // target/destination side vars
   // These variables keep track of the state of the current drag.
@@ -131,10 +162,12 @@ class nsDragService final : public nsBaseDragService, public nsIObserver {
   // We cache all data for the current drag context,
   // because waiting for the data in GetTargetDragData can be very slow.
   nsTHashMap<nsCStringHashKey, nsTArray<uint8_t>> mCachedData;
+  // mCachedData are tied to mCachedDragContext. mCachedDragContext is not
+  // ref counted and may be already deleted on Gtk side.
+  // We used it for mCachedData invalidation only and can't be used for
+  // any D&D operation.
+  uintptr_t mCachedDragContext;
 
-#ifdef MOZ_WAYLAND
-  RefPtr<nsWaylandDragContext> mPendingWaylandDragContext;
-#endif
   guint mPendingTime;
 
   // mTargetWindow and mTargetWindowPoint record the position of the last
@@ -146,15 +179,16 @@ class nsDragService final : public nsBaseDragService, public nsIObserver {
   // motion or drop events.  mTime records the corresponding timestamp.
   RefPtr<GtkWidget> mTargetWidget;
   RefPtr<GdkDragContext> mTargetDragContext;
-#ifdef MOZ_WAYLAND
-  RefPtr<nsWaylandDragContext> mTargetWaylandDragContext;
-#endif
-  // mTargetDragContextForRemote is set while waiting for a reply from
-  // a child process.
+
+  // When we route D'n'D request to child process
+  // (by EventStateManager::DispatchCrossProcessEvent)
+  // we save GdkDragContext to mTargetDragContextForRemote.
+  // When we get a reply from child process we use
+  // the stored GdkDragContext to send reply to OS.
+  //
+  // We need to store GdkDragContext because mTargetDragContext is cleared
+  // after every D'n'D event.
   RefPtr<GdkDragContext> mTargetDragContextForRemote;
-#ifdef MOZ_WAYLAND
-  RefPtr<nsWaylandDragContext> mTargetWaylandDragContextForRemote;
-#endif
   guint mTargetTime;
 
   // is it OK to drop on us?
@@ -169,9 +203,12 @@ class nsDragService final : public nsBaseDragService, public nsIObserver {
   bool IsTargetContextList(void);
   // this will get the native data from the last target given a
   // specific flavor
-  void GetTargetDragData(GdkAtom aFlavor);
+  void GetTargetDragData(GdkAtom aFlavor, nsTArray<nsCString>& aDropFlavors);
   // this will reset all of the target vars
   void TargetResetData(void);
+  // Ensure our data cache belongs to aDragContext and clear the cache if
+  // aDragContext is different than mCachedDragContext.
+  void EnsureCachedDataValidForContext(GdkDragContext* aDragContext);
 
   // source side vars
 
@@ -191,20 +228,37 @@ class nsDragService final : public nsBaseDragService, public nsIObserver {
 
   gboolean Schedule(DragTask aTask, nsWindow* aWindow,
                     GdkDragContext* aDragContext,
-                    nsWaylandDragContext* aPendingWaylandDragContext,
                     mozilla::LayoutDeviceIntPoint aWindowPoint, guint aTime);
 
   // Callback for g_idle_add_full() to run mScheduledTask.
   MOZ_CAN_RUN_SCRIPT static gboolean TaskDispatchCallback(gpointer data);
   MOZ_CAN_RUN_SCRIPT gboolean RunScheduledTask();
-  void UpdateDragAction();
   MOZ_CAN_RUN_SCRIPT void DispatchMotionEvents();
-  void ReplyToDragMotion(GdkDragContext* aDragContext);
-#ifdef MOZ_WAYLAND
-  void ReplyToDragMotion(nsWaylandDragContext* aDragContext);
+  void ReplyToDragMotion(GdkDragContext* aDragContext, guint aTime);
+  void ReplyToDragMotion();
+  void UpdateDragAction(GdkDragContext* aDragContext);
+  void UpdateDragAction();
+
+#ifdef MOZ_LOGGING
+  const char* GetDragServiceTaskName(nsDragService::DragTask aTask);
 #endif
+  void GetDragFlavors(nsTArray<nsCString>& aFlavors);
   gboolean DispatchDropEvent();
   static uint32_t GetCurrentModifiers();
+
+  nsresult CreateTempFile(nsITransferable* aItem, nsACString& aURI);
+  bool RemoveTempFiles();
+  static gboolean TaskRemoveTempFiles(gpointer data);
+
+  // the url of the temporary file that has been created in the current drag
+  // session
+  nsTArray<nsCString> mTempFileUrls;
+  // stores all temporary files
+  nsCOMArray<nsIFile> mTemporaryFiles;
+  // timer to trigger deletion of temporary files
+  guint mTempFileTimerID;
+  // How deep we're nested in event loops
+  int mEventLoopDepth;
 };
 
 #endif  // nsDragService_h__
