@@ -35,13 +35,11 @@ class nsLookAndFeel final : public nsXPLookAndFeel {
 
   bool GetDefaultDrawInTitlebar() override;
 
-  void GetGtkContentTheme(LookAndFeelTheme&) override;
   void GetThemeInfo(nsACString&) override;
 
-  static void ConfigureTheme(const LookAndFeelTheme& aTheme);
-
   static const nscolor kBlack = NS_RGB(0, 0, 0);
   static const nscolor kWhite = NS_RGB(255, 255, 255);
+  void OnColorSchemeSettingChanged();
 
  protected:
   static bool WidgetUsesImage(WidgetNodeType aNodeType);
@@ -76,8 +74,6 @@ class nsLookAndFeel final : public nsXPLookAndFeel {
     nscolor mInfoBackground = kWhite;
     nscolor mInfoText = kBlack;
     nscolor mMenuBackground = kWhite;
-    nscolor mMenuBarText = kBlack;
-    nscolor mMenuBarHoverText = kBlack;
     nscolor mMenuText = kBlack;
     nscolor mMenuTextInactive = kWhite;
     nscolor mMenuHover = kWhite;
@@ -105,7 +101,7 @@ class nsLookAndFeel final : public nsXPLookAndFeel {
     nscolor mTextSelectedText = kBlack;
     nscolor mTextSelectedBackground = kWhite;
     nscolor mAccentColor = kWhite;
-    nscolor mAccentColorForeground = kWhite;
+    nscolor mAccentColorText = kWhite;
     nscolor mSelectedItem = kWhite;
     nscolor mSelectedItemText = kBlack;
     nscolor mMozColHeaderText = kBlack;
@@ -123,7 +119,6 @@ class nsLookAndFeel final : public nsXPLookAndFeel {
 
     float mCaretRatio = 0.0f;
     int32_t mTitlebarRadius = 0;
-    int32_t mMenuRadius = 0;
     char16_t mInvisibleCharacter = 0;
     bool mMenuSupportsDrag = false;
 
