@@ -49,6 +49,7 @@
 #include "nsIMutableArray.h"
 #include "gfxContext.h"
 #include "gfxPlatform.h"
+#include "nscore.h"
 #include <algorithm>
 
 using namespace mozilla;
@@ -321,7 +322,7 @@ nsBaseDragService::InvokeDragSession(
   mIsDraggingTextInTextControl =
       mSourceNode->IsInNativeAnonymousSubtree() &&
       TextControlElement::FromNodeOrNull(
-          mSourceNode->GetClosestNativeAnonymousSubtreeRootParent());
+          mSourceNode->GetClosestNativeAnonymousSubtreeRootParentOrHost());
   mContentPolicyType = aContentPolicyType;
   mEndDragPoint = LayoutDeviceIntPoint(0, 0);
 
@@ -595,13 +596,18 @@ nsBaseDragService::EndDragSession(bool aDoneDrag, uint32_t aKeyModifiers) {
   if (mDragPopup) {
     nsXULPopupManager* pm = nsXULPopupManager::GetInstance();
     if (pm) {
-      pm->HidePopup(mDragPopup, false, true, false, false);
+      pm->HidePopup(mDragPopup, {HidePopupOption::DeselectMenu});
     }
   }
 
+  uint32_t dropEffect = nsIDragService::DRAGDROP_ACTION_NONE;
+  if (mDataTransfer) {
+    dropEffect = mDataTransfer->DropEffectInt();
+  }
+
   for (uint32_t i = 0; i < mChildProcesses.Length(); ++i) {
     mozilla::Unused << mChildProcesses[i]->SendEndDragSession(
-        aDoneDrag, mUserCancelled, mEndDragPoint, aKeyModifiers);
+        aDoneDrag, mUserCancelled, mEndDragPoint, aKeyModifiers, dropEffect);
     // Continue sending input events with input priority when stopping the dnd
     // session.
     mChildProcesses[i]->SetInputPriorityEventEnabled(true);
@@ -848,7 +854,7 @@ nsresult nsBaseDragService::DrawDrag(nsINode* aDOMNode,
 
     nsIFrame* frame = content->GetPrimaryFrame();
     if (frame && frame->IsMenuPopupFrame()) {
-      mDragPopup = content;
+      mDragPopup = content->AsElement();
     }
   }
 
@@ -935,14 +941,12 @@ nsresult nsBaseDragService::DrawDragForImage(
             destSize, SurfaceFormat::B8G8R8A8);
     if (!dt || !dt->IsValid()) return NS_ERROR_FAILURE;
 
-    RefPtr<gfxContext> ctx = gfxContext::CreateOrNull(dt);
-    if (!ctx) return NS_ERROR_FAILURE;
+    gfxContext ctx(dt);
 
-    ImgDrawResult res =
-        imgContainer->Draw(ctx, destSize, ImageRegion::Create(destSize),
-                           imgIContainer::FRAME_CURRENT, SamplingFilter::GOOD,
-                           /* no SVGImageContext */ Nothing(),
-                           imgIContainer::FLAG_SYNC_DECODE, 1.0);
+    ImgDrawResult res = imgContainer->Draw(
+        &ctx, destSize, ImageRegion::Create(destSize),
+        imgIContainer::FRAME_CURRENT, SamplingFilter::GOOD, SVGImageContext(),
+        imgIContainer::FLAG_SYNC_DECODE, 1.0);
     if (res == ImgDrawResult::BAD_IMAGE || res == ImgDrawResult::BAD_ARGS ||
         res == ImgDrawResult::NOT_SUPPORTED) {
       return NS_ERROR_FAILURE;
@@ -1010,8 +1014,22 @@ bool nsBaseDragService::MaybeAddChildProcess(
 bool nsBaseDragService::RemoveAllChildProcesses() {
   for (uint32_t c = 0; c < mChildProcesses.Length(); c++) {
     mozilla::Unused << mChildProcesses[c]->SendEndDragSession(
-        true, false, LayoutDeviceIntPoint(), 0);
+        true, false, LayoutDeviceIntPoint(), 0,
+        nsIDragService::DRAGDROP_ACTION_NONE);
   }
   mChildProcesses.Clear();
   return true;
 }
+
+NS_IMETHODIMP
+nsBaseDragService::MaybeEditorDeletedSourceNode(Element* aEditingHost) {
+  // If builtin editor of Blink and WebKit deletes the source node,they retarget
+  // the source node to the editing host.
+  // https://source.chromium.org/chromium/chromium/src/+/main:third_party/blink/renderer/core/page/drag_controller.cc;l=724;drc=d9ba13b8cd8ac0faed7afc3d1f7e4b67ebac2a0b
+  // That allows editor apps listens to "dragend" event in editing host or its
+  // ancestors.  Therefore, we should follow them for compatibility.
+  if (mSourceNode && !mSourceNode->IsInComposedDoc()) {
+    mSourceNode = aEditingHost;
+  }
+  return NS_OK;
+}
