@@ -4,50 +4,27 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "nsPrintSettingsImpl.h"
+
+#include "prenv.h"
+#include "nsCoord.h"
+#include "nsPaper.h"
 #include "nsReadableUtils.h"
-#include "nsIPrintSession.h"
+#include "mozilla/DebugOnly.h"
 #include "mozilla/RefPtr.h"
 
+using namespace mozilla;
+
 #define DEFAULT_MARGIN_WIDTH 0.5
 
 NS_IMPL_ISUPPORTS(nsPrintSettings, nsIPrintSettings)
 
-nsPrintSettings::nsPrintSettings()
-    : mPrintOptions(0L),
-      mPrintRange(kRangeAllPages),
-      mStartPageNum(1),
-      mEndPageNum(1),
-      mScaling(1.0),
-      mPrintBGColors(false),
-      mPrintBGImages(false),
-      mIsCancelled(false),
-      mSaveOnCancel(true),
-      mPrintSilent(false),
-      mShrinkToFit(true),
-      mShowPrintProgress(true),
-      mPrintPageDelay(50),
-      mPaperData(0),
-      mPaperWidth(8.5),
-      mPaperHeight(11.0),
-      mPaperSizeUnit(kPaperSizeInches),
-      mPrintReversed(false),
-      mPrintInColor(true),
-      mOrientation(kPortraitOrientation),
-      mResolution(0),
-      mDuplex(0),
-      mNumCopies(1),
-      mPrintToFile(false),
-      mOutputFormat(kOutputFormatNative),
-      mIsInitedFromPrinter(false),
-      mIsInitedFromPrefs(false) {
+nsPrintSettings::nsPrintSettings() {
   /* member initializers and constructor code */
   int32_t marginWidth = NS_INCHES_TO_INT_TWIPS(DEFAULT_MARGIN_WIDTH);
   mMargin.SizeTo(marginWidth, marginWidth, marginWidth, marginWidth);
   mEdge.SizeTo(0, 0, 0, 0);
   mUnwriteableMargin.SizeTo(0, 0, 0, 0);
 
-  mPrintOptions = kPrintOddPages | kPrintEvenPages;
-
   mHeaderStrs[0].AssignLiteral("&T");
   mHeaderStrs[2].AssignLiteral("&U");
 
@@ -56,58 +33,56 @@ nsPrintSettings::nsPrintSettings()
   mFooterStrs[2].AssignLiteral("&D");
 }
 
+void nsPrintSettings::InitWithInitializer(
+    const PrintSettingsInitializer& aSettings) {
+  const double kInchesPerPoint = 1.0 / 72.0;
+
+  SetPrinterName(aSettings.mPrinter);
+  SetPrintInColor(aSettings.mPrintInColor);
+  SetResolution(aSettings.mResolution);
+  SetNumCopies(aSettings.mNumCopies);
+  SetDuplex(aSettings.mDuplex);
+  // The paper ID used by nsPrintSettings is the non-localizable identifier
+  // exposed as "id" by the paper, not the potentially localized human-friendly
+  // "name", which could change, e.g. if the user changes their system locale.
+  SetPaperId(aSettings.mPaperInfo.mId);
+
+  // Set the paper sizes to match the unit.
+  SetPaperSizeUnit(aSettings.mPaperSizeUnit);
+  double sizeUnitsPerPoint =
+      aSettings.mPaperSizeUnit == kPaperSizeInches ? 1.0 / 72.0 : 25.4 / 72.0;
+  SetPaperWidth(aSettings.mPaperInfo.mSize.width * sizeUnitsPerPoint);
+  SetPaperHeight(aSettings.mPaperInfo.mSize.height * sizeUnitsPerPoint);
+
+  // If our initializer says that we're producing portrait-mode sheets of
+  // paper, then our page format must also be portrait-mode; unless we've got
+  // a pages-per-sheet value with orthogonal pages/sheets, in which case it's
+  // reversed.
+  const bool areSheetsOfPaperPortraitMode =
+      (aSettings.mSheetOrientation == kPortraitOrientation);
+  const bool arePagesPortraitMode =
+      (areSheetsOfPaperPortraitMode != HasOrthogonalSheetsAndPages());
+  SetOrientation(arePagesPortraitMode ? kPortraitOrientation
+                                      : kLandscapeOrientation);
+
+  if (aSettings.mPaperInfo.mUnwriteableMargin) {
+    const auto& margin = aSettings.mPaperInfo.mUnwriteableMargin.value();
+    // Margins are stored internally in TWIPS, but the setters expect inches.
+    SetUnwriteableMarginTop(margin.top * kInchesPerPoint);
+    SetUnwriteableMarginRight(margin.right * kInchesPerPoint);
+    SetUnwriteableMarginBottom(margin.bottom * kInchesPerPoint);
+    SetUnwriteableMarginLeft(margin.left * kInchesPerPoint);
+  }
+
+  // Set this last because other setters may overwrite its value.
+  SetIsInitializedFromPrinter(true);
+}
+
 nsPrintSettings::nsPrintSettings(const nsPrintSettings& aPS) { *this = aPS; }
 
 nsPrintSettings::~nsPrintSettings() = default;
 
-NS_IMETHODIMP nsPrintSettings::GetPrintSession(
-    nsIPrintSession** aPrintSession) {
-  NS_ENSURE_ARG_POINTER(aPrintSession);
-  *aPrintSession = nullptr;
-
-  nsCOMPtr<nsIPrintSession> session = do_QueryReferent(mSession);
-  if (!session) return NS_ERROR_NOT_INITIALIZED;
-  *aPrintSession = session;
-  NS_ADDREF(*aPrintSession);
-  return NS_OK;
-}
-NS_IMETHODIMP nsPrintSettings::SetPrintSession(nsIPrintSession* aPrintSession) {
-  // Clearing it by passing nullptr is not allowed. That's why we
-  // use a weak ref so that it doesn't have to be cleared.
-  NS_ENSURE_ARG(aPrintSession);
-
-  mSession = do_GetWeakReference(aPrintSession);
-  if (!mSession) {
-    // This may happen if the implementation of this object does
-    // not support weak references - programmer error.
-    NS_ERROR("Could not get a weak reference from aPrintSession");
-    return NS_ERROR_FAILURE;
-  }
-  return NS_OK;
-}
-
-NS_IMETHODIMP nsPrintSettings::GetStartPageRange(int32_t* aStartPageRange) {
-  // NS_ENSURE_ARG_POINTER(aStartPageRange);
-  *aStartPageRange = mStartPageNum;
-  return NS_OK;
-}
-NS_IMETHODIMP nsPrintSettings::SetStartPageRange(int32_t aStartPageRange) {
-  mStartPageNum = aStartPageRange;
-  return NS_OK;
-}
-
-NS_IMETHODIMP nsPrintSettings::GetEndPageRange(int32_t* aEndPageRange) {
-  // NS_ENSURE_ARG_POINTER(aEndPageRange);
-  *aEndPageRange = mEndPageNum;
-  return NS_OK;
-}
-NS_IMETHODIMP nsPrintSettings::SetEndPageRange(int32_t aEndPageRange) {
-  mEndPageNum = aEndPageRange;
-  return NS_OK;
-}
-
 NS_IMETHODIMP nsPrintSettings::GetPrintReversed(bool* aPrintReversed) {
-  // NS_ENSURE_ARG_POINTER(aPrintReversed);
   *aPrintReversed = mPrintReversed;
   return NS_OK;
 }
@@ -117,7 +92,6 @@ NS_IMETHODIMP nsPrintSettings::SetPrintReversed(bool aPrintReversed) {
 }
 
 NS_IMETHODIMP nsPrintSettings::GetPrintInColor(bool* aPrintInColor) {
-  // NS_ENSURE_ARG_POINTER(aPrintInColor);
   *aPrintInColor = mPrintInColor;
   return NS_OK;
 }
@@ -127,7 +101,6 @@ NS_IMETHODIMP nsPrintSettings::SetPrintInColor(bool aPrintInColor) {
 }
 
 NS_IMETHODIMP nsPrintSettings::GetOrientation(int32_t* aOrientation) {
-  NS_ENSURE_ARG_POINTER(aOrientation);
   *aOrientation = mOrientation;
   return NS_OK;
 }
@@ -181,13 +154,35 @@ NS_IMETHODIMP nsPrintSettings::SetNumCopies(int32_t aNumCopies) {
   return NS_OK;
 }
 
-NS_IMETHODIMP nsPrintSettings::GetPrintToFile(bool* aPrintToFile) {
-  // NS_ENSURE_ARG_POINTER(aPrintToFile);
-  *aPrintToFile = mPrintToFile;
+NS_IMETHODIMP nsPrintSettings::GetNumPagesPerSheet(int32_t* aNumPagesPerSheet) {
+  NS_ENSURE_ARG_POINTER(aNumPagesPerSheet);
+  *aNumPagesPerSheet = mNumPagesPerSheet;
   return NS_OK;
 }
-NS_IMETHODIMP nsPrintSettings::SetPrintToFile(bool aPrintToFile) {
-  mPrintToFile = aPrintToFile;
+NS_IMETHODIMP nsPrintSettings::SetNumPagesPerSheet(int32_t aNumPagesPerSheet) {
+  mNumPagesPerSheet = aNumPagesPerSheet;
+  return NS_OK;
+}
+
+NS_IMETHODIMP nsPrintSettings::GetOutputDestination(
+    OutputDestinationType* aDestination) {
+  *aDestination = mOutputDestination;
+  return NS_OK;
+}
+
+NS_IMETHODIMP nsPrintSettings::SetOutputDestination(
+    OutputDestinationType aDestination) {
+  mOutputDestination = aDestination;
+  return NS_OK;
+}
+
+NS_IMETHODIMP nsPrintSettings::SetOutputStream(nsIOutputStream* aStream) {
+  mOutputStream = aStream;
+  return NS_OK;
+}
+
+NS_IMETHODIMP nsPrintSettings::GetOutputStream(nsIOutputStream** aStream) {
+  NS_IF_ADDREF(*aStream = mOutputStream.get());
   return NS_OK;
 }
 
@@ -222,24 +217,24 @@ NS_IMETHODIMP nsPrintSettings::SetPrintPageDelay(int32_t aPrintPageDelay) {
 NS_IMETHODIMP nsPrintSettings::GetIsInitializedFromPrinter(
     bool* aIsInitializedFromPrinter) {
   NS_ENSURE_ARG_POINTER(aIsInitializedFromPrinter);
-  *aIsInitializedFromPrinter = (bool)mIsInitedFromPrinter;
+  *aIsInitializedFromPrinter = mIsInitedFromPrinter;
   return NS_OK;
 }
 NS_IMETHODIMP nsPrintSettings::SetIsInitializedFromPrinter(
     bool aIsInitializedFromPrinter) {
-  mIsInitedFromPrinter = (bool)aIsInitializedFromPrinter;
+  mIsInitedFromPrinter = aIsInitializedFromPrinter;
   return NS_OK;
 }
 
 NS_IMETHODIMP nsPrintSettings::GetIsInitializedFromPrefs(
     bool* aInitializedFromPrefs) {
   NS_ENSURE_ARG_POINTER(aInitializedFromPrefs);
-  *aInitializedFromPrefs = (bool)mIsInitedFromPrefs;
+  *aInitializedFromPrefs = mIsInitedFromPrefs;
   return NS_OK;
 }
 NS_IMETHODIMP nsPrintSettings::SetIsInitializedFromPrefs(
     bool aInitializedFromPrefs) {
-  mIsInitedFromPrefs = (bool)aInitializedFromPrefs;
+  mIsInitedFromPrefs = aInitializedFromPrefs;
   return NS_OK;
 }
 
@@ -411,16 +406,6 @@ NS_IMETHODIMP nsPrintSettings::SetPrintBGImages(bool aPrintBGImages) {
   return NS_OK;
 }
 
-NS_IMETHODIMP nsPrintSettings::GetPrintRange(int16_t* aPrintRange) {
-  NS_ENSURE_ARG_POINTER(aPrintRange);
-  *aPrintRange = mPrintRange;
-  return NS_OK;
-}
-NS_IMETHODIMP nsPrintSettings::SetPrintRange(int16_t aPrintRange) {
-  mPrintRange = aPrintRange;
-  return NS_OK;
-}
-
 NS_IMETHODIMP nsPrintSettings::GetTitle(nsAString& aTitle) {
   aTitle = mTitle;
   return NS_OK;
@@ -439,47 +424,6 @@ NS_IMETHODIMP nsPrintSettings::SetDocURL(const nsAString& aDocURL) {
   return NS_OK;
 }
 
-/** ---------------------------------------------------
- *  See documentation in nsPrintSettingsImpl.h
- *	@update 1/12/01 rods
- */
-NS_IMETHODIMP
-nsPrintSettings::GetPrintOptions(int32_t aType, bool* aTurnOnOff) {
-  NS_ENSURE_ARG_POINTER(aTurnOnOff);
-  *aTurnOnOff = mPrintOptions & aType ? true : false;
-  return NS_OK;
-}
-/** ---------------------------------------------------
- *  See documentation in nsPrintSettingsImpl.h
- *	@update 1/12/01 rods
- */
-NS_IMETHODIMP
-nsPrintSettings::SetPrintOptions(int32_t aType, bool aTurnOnOff) {
-  if (aTurnOnOff) {
-    mPrintOptions |= aType;
-  } else {
-    mPrintOptions &= ~aType;
-  }
-  return NS_OK;
-}
-
-/** ---------------------------------------------------
- *  See documentation in nsPrintSettingsImpl.h
- *	@update 1/12/01 rods
- */
-NS_IMETHODIMP
-nsPrintSettings::GetPrintOptionsBits(int32_t* aBits) {
-  NS_ENSURE_ARG_POINTER(aBits);
-  *aBits = mPrintOptions;
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsPrintSettings::SetPrintOptionsBits(int32_t aBits) {
-  mPrintOptions = aBits;
-  return NS_OK;
-}
-
 NS_IMETHODIMP nsPrintSettings::GetHeaderStrLeft(nsAString& aTitle) {
   aTitle = mHeaderStrs[0];
   return NS_OK;
@@ -554,37 +498,62 @@ NS_IMETHODIMP nsPrintSettings::SetShrinkToFit(bool aShrinkToFit) {
   return NS_OK;
 }
 
-NS_IMETHODIMP nsPrintSettings::GetShowPrintProgress(bool* aShowPrintProgress) {
-  NS_ENSURE_ARG_POINTER(aShowPrintProgress);
-  *aShowPrintProgress = mShowPrintProgress;
-  return NS_OK;
-}
-NS_IMETHODIMP nsPrintSettings::SetShowPrintProgress(bool aShowPrintProgress) {
-  mShowPrintProgress = aShowPrintProgress;
+NS_IMETHODIMP nsPrintSettings::GetShowMarginGuides(bool* aShowMarginGuides) {
+  *aShowMarginGuides = mShowMarginGuides;
   return NS_OK;
 }
 
-NS_IMETHODIMP nsPrintSettings::GetPaperName(nsAString& aPaperName) {
-  aPaperName = mPaperName;
-  return NS_OK;
-}
-NS_IMETHODIMP nsPrintSettings::SetPaperName(const nsAString& aPaperName) {
-  mPaperName = aPaperName;
+NS_IMETHODIMP nsPrintSettings::SetShowMarginGuides(bool aShowMarginGuides) {
+  mShowMarginGuides = aShowMarginGuides;
   return NS_OK;
 }
 
-NS_IMETHODIMP nsPrintSettings::GetIsCancelled(bool* aIsCancelled) {
-  NS_ENSURE_ARG_POINTER(aIsCancelled);
-  *aIsCancelled = mIsCancelled;
-  return NS_OK;
-}
-NS_IMETHODIMP nsPrintSettings::SetIsCancelled(bool aIsCancelled) {
-  mIsCancelled = aIsCancelled;
+NS_IMETHODIMP nsPrintSettings::GetHonorPageRuleMargins(bool* aResult) {
+  *aResult = mHonorPageRuleMargins;
   return NS_OK;
 }
 
-NS_IMETHODIMP nsPrintSettings::GetSaveOnCancel(bool* aSaveOnCancel) {
-  *aSaveOnCancel = mSaveOnCancel;
+NS_IMETHODIMP nsPrintSettings::SetHonorPageRuleMargins(bool aHonor) {
+  mHonorPageRuleMargins = aHonor;
+  return NS_OK;
+}
+
+NS_IMETHODIMP nsPrintSettings::GetUsePageRuleSizeAsPaperSize(bool* aResult) {
+  *aResult = mUsePageRuleSizeAsPaperSize;
+  return NS_OK;
+}
+
+NS_IMETHODIMP nsPrintSettings::SetUsePageRuleSizeAsPaperSize(bool aHonor) {
+  mUsePageRuleSizeAsPaperSize = aHonor;
+  return NS_OK;
+}
+
+NS_IMETHODIMP nsPrintSettings::GetIgnoreUnwriteableMargins(bool* aResult) {
+  *aResult = mIgnoreUnwriteableMargins;
+  return NS_OK;
+}
+
+NS_IMETHODIMP nsPrintSettings::SetIgnoreUnwriteableMargins(bool aIgnore) {
+  mIgnoreUnwriteableMargins = aIgnore;
+  return NS_OK;
+}
+
+NS_IMETHODIMP nsPrintSettings::GetPrintSelectionOnly(bool* aResult) {
+  *aResult = mPrintSelectionOnly;
+  return NS_OK;
+}
+
+NS_IMETHODIMP nsPrintSettings::SetPrintSelectionOnly(bool aSelectionOnly) {
+  mPrintSelectionOnly = aSelectionOnly;
+  return NS_OK;
+}
+
+NS_IMETHODIMP nsPrintSettings::GetPaperId(nsAString& aPaperId) {
+  aPaperId = mPaperId;
+  return NS_OK;
+}
+NS_IMETHODIMP nsPrintSettings::SetPaperId(const nsAString& aPaperId) {
+  mPaperId = aPaperId;
   return NS_OK;
 }
 
@@ -618,16 +587,6 @@ NS_IMETHODIMP nsPrintSettings::SetPaperSizeUnit(int16_t aPaperSizeUnit) {
   return NS_OK;
 }
 
-NS_IMETHODIMP nsPrintSettings::GetPaperData(int16_t* aPaperData) {
-  NS_ENSURE_ARG_POINTER(aPaperData);
-  *aPaperData = mPaperData;
-  return NS_OK;
-}
-NS_IMETHODIMP nsPrintSettings::SetPaperData(int16_t aPaperData) {
-  mPaperData = aPaperData;
-  return NS_OK;
-}
-
 /** ---------------------------------------------------
  *  See documentation in nsPrintSettingsService.h
  *	@update 6/21/00 dwc
@@ -665,34 +624,14 @@ nsPrintSettings::SetUnwriteableMarginInTwips(nsIntMargin& aUnwriteableMargin) {
   return NS_OK;
 }
 
-/** ---------------------------------------------------
- *  See documentation in nsPrintSettingsService.h
- *	@update 6/21/00 dwc
- */
-NS_IMETHODIMP
-nsPrintSettings::GetMarginInTwips(nsIntMargin& aMargin) {
-  aMargin = mMargin;
-  return NS_OK;
-}
+nsIntMargin nsPrintSettings::GetMarginInTwips() { return mMargin; }
 
-NS_IMETHODIMP
-nsPrintSettings::GetEdgeInTwips(nsIntMargin& aEdge) {
-  aEdge = mEdge;
-  return NS_OK;
-}
+nsIntMargin nsPrintSettings::GetEdgeInTwips() { return mEdge; }
 
-NS_IMETHODIMP
-nsPrintSettings::GetUnwriteableMarginInTwips(nsIntMargin& aUnwriteableMargin) {
-  aUnwriteableMargin = mUnwriteableMargin;
-  return NS_OK;
+nsIntMargin nsPrintSettings::GetUnwriteableMarginInTwips() {
+  return mUnwriteableMargin;
 }
 
-/** ---------------------------------------------------
- * Stub - platform-specific implementations can use this function.
- */
-NS_IMETHODIMP
-nsPrintSettings::SetupSilentPrinting() { return NS_OK; }
-
 /** ---------------------------------------------------
  *  See documentation in nsPrintSettingsService.h
  */
@@ -702,6 +641,8 @@ nsPrintSettings::GetEffectivePageSize(double* aWidth, double* aHeight) {
     *aWidth = NS_INCHES_TO_TWIPS(float(mPaperWidth));
     *aHeight = NS_INCHES_TO_TWIPS(float(mPaperHeight));
   } else {
+    MOZ_ASSERT(mPaperSizeUnit == kPaperSizeMillimeters,
+               "unexpected paper size unit");
     *aWidth = NS_MILLIMETERS_TO_TWIPS(float(mPaperWidth));
     *aHeight = NS_MILLIMETERS_TO_TWIPS(float(mPaperHeight));
   }
@@ -713,12 +654,64 @@ nsPrintSettings::GetEffectivePageSize(double* aWidth, double* aHeight) {
   return NS_OK;
 }
 
+bool nsPrintSettings::HasOrthogonalSheetsAndPages() {
+  return mNumPagesPerSheet == 2 || mNumPagesPerSheet == 6;
+}
+
+void nsPrintSettings::GetEffectiveSheetSize(double* aWidth, double* aHeight) {
+  mozilla::DebugOnly<nsresult> rv = GetEffectivePageSize(aWidth, aHeight);
+
+  // Our GetEffectivePageSize impls only return NS_OK, so this should hold:
+  MOZ_ASSERT(NS_SUCCEEDED(rv), "Uh oh, GetEffectivePageSize failed");
+
+  if (HasOrthogonalSheetsAndPages()) {
+    std::swap(*aWidth, *aHeight);
+  }
+}
+
+int32_t nsPrintSettings::GetSheetOrientation() {
+  if (HasOrthogonalSheetsAndPages()) {
+    // Sheet orientation is rotated with respect to the page orientation.
+    return kLandscapeOrientation == mOrientation ? kPortraitOrientation
+                                                 : kLandscapeOrientation;
+  }
+
+  // Sheet orientation is the same as the page orientation.
+  return mOrientation;
+}
+
+NS_IMETHODIMP
+nsPrintSettings::SetPageRanges(const nsTArray<int32_t>& aPages) {
+  // Needs to be a set of (start, end) pairs.
+  if (aPages.Length() % 2 != 0) {
+    return NS_ERROR_FAILURE;
+  }
+  mPageRanges = aPages.Clone();
+  return NS_OK;
+}
+
 NS_IMETHODIMP
 nsPrintSettings::GetPageRanges(nsTArray<int32_t>& aPages) {
-  aPages.Clear();
+  aPages = mPageRanges.Clone();
   return NS_OK;
 }
 
+bool nsIPrintSettings::IsPageSkipped(int32_t aPageNum,
+                                     const nsTArray<int32_t>& aRanges) {
+  MOZ_RELEASE_ASSERT(aRanges.Length() % 2 == 0);
+  if (aRanges.IsEmpty()) {
+    return false;
+  }
+  for (size_t i = 0; i < aRanges.Length(); i += 2) {
+    if (aRanges[i] <= aPageNum && aPageNum <= aRanges[i + 1]) {
+      // The page is included in this piece of the custom range,
+      // so it's not skipped.
+      return false;
+    }
+  }
+  return true;
+}
+
 nsresult nsPrintSettings::_Clone(nsIPrintSettings** _retval) {
   RefPtr<nsPrintSettings> printSettings = new nsPrintSettings(*this);
   printSettings.forget(_retval);
@@ -737,6 +730,140 @@ nsresult nsPrintSettings::_Assign(nsIPrintSettings* aPS) {
   return NS_OK;
 }
 
+nsresult nsPrintSettings::EquivalentTo(nsIPrintSettings* aPrintSettings,
+                                       bool* _retval) {
+  MOZ_ASSERT(aPrintSettings);
+  *_retval = false;
+  auto* other = static_cast<nsPrintSettings*>(aPrintSettings);
+  if (GetMarginInTwips() != aPrintSettings->GetMarginInTwips()) {
+    return NS_OK;
+  }
+  if (GetEdgeInTwips() != aPrintSettings->GetEdgeInTwips()) {
+    return NS_OK;
+  }
+  if (GetUnwriteableMarginInTwips() !=
+      aPrintSettings->GetUnwriteableMarginInTwips()) {
+    return NS_OK;
+  }
+  nsTArray<int32_t> ourPageRanges, otherPageRanges;
+  if (NS_FAILED(GetPageRanges(ourPageRanges)) ||
+      NS_FAILED(aPrintSettings->GetPageRanges(otherPageRanges)) ||
+      ourPageRanges != otherPageRanges) {
+    return NS_OK;
+  }
+  double ourScaling, otherScaling;
+  if (NS_FAILED(GetScaling(&ourScaling)) ||
+      NS_FAILED(aPrintSettings->GetScaling(&otherScaling)) ||
+      ourScaling != otherScaling) {
+    return NS_OK;
+  }
+  if (GetPrintBGColors() != aPrintSettings->GetPrintBGColors()) {
+    return NS_OK;
+  }
+  if (GetPrintBGImages() != aPrintSettings->GetPrintBGImages()) {
+    return NS_OK;
+  }
+  if (GetPrintSelectionOnly() != aPrintSettings->GetPrintSelectionOnly()) {
+    return NS_OK;
+  }
+  if (GetShrinkToFit() != aPrintSettings->GetShrinkToFit()) {
+    return NS_OK;
+  }
+  if (GetShowMarginGuides() != aPrintSettings->GetShowMarginGuides()) {
+    return NS_OK;
+  }
+  if (GetHonorPageRuleMargins() != aPrintSettings->GetHonorPageRuleMargins()) {
+    return NS_OK;
+  }
+  if (GetUsePageRuleSizeAsPaperSize() !=
+      aPrintSettings->GetUsePageRuleSizeAsPaperSize()) {
+    return NS_OK;
+  }
+  if (GetIgnoreUnwriteableMargins() !=
+      aPrintSettings->GetIgnoreUnwriteableMargins()) {
+    return NS_OK;
+  }
+  nsAutoString ourTitle, otherTitle;
+  if (NS_FAILED(GetTitle(ourTitle)) ||
+      NS_FAILED(aPrintSettings->GetTitle(otherTitle)) ||
+      ourTitle != otherTitle) {
+    return NS_OK;
+  }
+  nsAutoString ourUrl, otherUrl;
+  if (NS_FAILED(GetDocURL(ourUrl)) ||
+      NS_FAILED(aPrintSettings->GetDocURL(otherUrl)) || ourUrl != otherUrl) {
+    return NS_OK;
+  }
+  if (!mozilla::ArrayEqual(mHeaderStrs, other->mHeaderStrs) ||
+      !mozilla::ArrayEqual(mFooterStrs, other->mFooterStrs)) {
+    return NS_OK;
+  }
+  nsAutoString ourPaperId, otherPaperId;
+  if (NS_FAILED(GetPaperId(ourPaperId)) ||
+      NS_FAILED(aPrintSettings->GetPaperId(otherPaperId)) ||
+      ourPaperId != otherPaperId) {
+    return NS_OK;
+  }
+  double ourWidth, ourHeight, otherWidth, otherHeight;
+  if (NS_FAILED(GetEffectivePageSize(&ourWidth, &ourHeight)) ||
+      NS_FAILED(other->GetEffectivePageSize(&otherWidth, &otherHeight)) ||
+      std::abs(ourWidth - otherWidth) >= 1 ||
+      std::abs(ourHeight - otherHeight) >= 1) {
+    return NS_OK;
+  }
+  int32_t ourOrientation, otherOrientation;
+  if (NS_FAILED(GetOrientation(&ourOrientation)) ||
+      NS_FAILED(aPrintSettings->GetOrientation(&otherOrientation)) ||
+      ourOrientation != otherOrientation) {
+    return NS_OK;
+  }
+  int32_t ourResolution, otherResolution;
+  if (NS_FAILED(GetResolution(&ourResolution)) ||
+      NS_FAILED(aPrintSettings->GetResolution(&otherResolution)) ||
+      ourResolution != otherResolution) {
+    return NS_OK;
+  }
+  int32_t ourNumPagesPerSheet, otherNumPagesPerSheet;
+  if (NS_FAILED(GetNumPagesPerSheet(&ourNumPagesPerSheet)) ||
+      NS_FAILED(aPrintSettings->GetNumPagesPerSheet(&otherNumPagesPerSheet)) ||
+      ourNumPagesPerSheet != otherNumPagesPerSheet) {
+    return NS_OK;
+  }
+
+  *_retval = true;
+  return NS_OK;
+}
+
+mozilla::PrintSettingsInitializer nsPrintSettings::GetSettingsInitializer() {
+  mozilla::PrintSettingsInitializer settingsInitializer;
+  settingsInitializer.mPrinter.Assign(mPrinter);
+  settingsInitializer.mPaperInfo.mId = mPaperId;
+
+  double pointsPerSizeUnit =
+      mPaperSizeUnit == kPaperSizeInches ? 72.0 : 72.0 / 25.4;
+  settingsInitializer.mPaperInfo.mSize = {mPaperWidth * pointsPerSizeUnit,
+                                          mPaperHeight * pointsPerSizeUnit};
+
+  // Unwritable margins are stored in TWIPS here and points in PaperInfo.
+  settingsInitializer.mPaperInfo.mUnwriteableMargin =
+      Some(mozilla::gfx::MarginDouble(
+          mUnwriteableMargin.top / 20.0, mUnwriteableMargin.right / 20.0,
+          mUnwriteableMargin.bottom / 20.0, mUnwriteableMargin.left / 20.0));
+
+  settingsInitializer.mPrintInColor = mPrintInColor;
+  settingsInitializer.mResolution = mResolution;
+  settingsInitializer.mSheetOrientation = GetSheetOrientation();
+  settingsInitializer.mNumCopies = mNumCopies;
+  settingsInitializer.mDuplex = mDuplex;
+  RefPtr<nsIPrintSettings> settingsToInitialize;
+  MOZ_ALWAYS_SUCCEEDS(Clone(getter_AddRefs(settingsToInitialize)));
+  settingsInitializer.mPrintSettings =
+      new nsMainThreadPtrHolder<nsPrintSettings>(
+          "PrintSettingsInitializer::mPrintSettings",
+          settingsToInitialize.forget().downcast<nsPrintSettings>());
+  return settingsInitializer;
+}
+
 NS_IMETHODIMP
 nsPrintSettings::Assign(nsIPrintSettings* aPS) {
   NS_ENSURE_ARG(aPS);
@@ -749,34 +876,40 @@ nsPrintSettings& nsPrintSettings::operator=(const nsPrintSettings& rhs) {
     return *this;
   }
 
-  mStartPageNum = rhs.mStartPageNum;
-  mEndPageNum = rhs.mEndPageNum;
+  mPageRanges = rhs.mPageRanges.Clone();
   mMargin = rhs.mMargin;
   mEdge = rhs.mEdge;
   mUnwriteableMargin = rhs.mUnwriteableMargin;
   mScaling = rhs.mScaling;
   mPrintBGColors = rhs.mPrintBGColors;
   mPrintBGImages = rhs.mPrintBGImages;
-  mPrintRange = rhs.mPrintRange;
   mTitle = rhs.mTitle;
   mURL = rhs.mURL;
-  mIsCancelled = rhs.mIsCancelled;
   mPrintSilent = rhs.mPrintSilent;
   mShrinkToFit = rhs.mShrinkToFit;
-  mShowPrintProgress = rhs.mShowPrintProgress;
-  mPaperName = rhs.mPaperName;
-  mPaperData = rhs.mPaperData;
+  mShowMarginGuides = rhs.mShowMarginGuides;
+  mHonorPageRuleMargins = rhs.mHonorPageRuleMargins;
+  mUsePageRuleSizeAsPaperSize = rhs.mUsePageRuleSizeAsPaperSize;
+  mIgnoreUnwriteableMargins = rhs.mIgnoreUnwriteableMargins;
+  mPrintSelectionOnly = rhs.mPrintSelectionOnly;
+  mPaperId = rhs.mPaperId;
   mPaperWidth = rhs.mPaperWidth;
   mPaperHeight = rhs.mPaperHeight;
   mPaperSizeUnit = rhs.mPaperSizeUnit;
   mPrintReversed = rhs.mPrintReversed;
   mPrintInColor = rhs.mPrintInColor;
   mOrientation = rhs.mOrientation;
+  mResolution = rhs.mResolution;
+  mDuplex = rhs.mDuplex;
   mNumCopies = rhs.mNumCopies;
+  mNumPagesPerSheet = rhs.mNumPagesPerSheet;
   mPrinter = rhs.mPrinter;
-  mPrintToFile = rhs.mPrintToFile;
+  mOutputDestination = rhs.mOutputDestination;
+  mOutputStream = rhs.mOutputStream;
   mToFileName = rhs.mToFileName;
   mOutputFormat = rhs.mOutputFormat;
+  mIsInitedFromPrinter = rhs.mIsInitedFromPrinter;
+  mIsInitedFromPrefs = rhs.mIsInitedFromPrefs;
   mPrintPageDelay = rhs.mPrintPageDelay;
 
   for (int32_t i = 0; i < NUM_HEAD_FOOT; i++) {
@@ -786,3 +919,23 @@ nsPrintSettings& nsPrintSettings::operator=(const nsPrintSettings& rhs) {
 
   return *this;
 }
+
+void nsPrintSettings::SetDefaultFileName() {
+  nsAutoString filename;
+  nsresult rv = GetToFileName(filename);
+  if (NS_FAILED(rv) || filename.IsEmpty()) {
+    const char* path = PR_GetEnv("PWD");
+    if (!path) {
+      path = PR_GetEnv("HOME");
+    }
+
+    if (path) {
+      CopyUTF8toUTF16(mozilla::MakeStringSpan(path), filename);
+      filename.AppendLiteral("/mozilla.pdf");
+    } else {
+      filename.AssignLiteral("mozilla.pdf");
+    }
+
+    SetToFileName(filename);
+  }
+}
