@@ -239,8 +239,7 @@ class nsAutoRetainUIKitObject {
 - (BOOL)isUsingMainThreadOpenGL {
   if (!mGeckoChild || ![self window]) return NO;
 
-  return mGeckoChild->GetLayerManager(nullptr)->GetBackendType() ==
-         mozilla::layers::LayersBackend::LAYERS_OPENGL;
+  return NO;
 }
 
 - (void)drawUsingOpenGL {
@@ -318,7 +317,7 @@ class nsAutoRetainUIKitObject {
   // Create Cairo objects.
   RefPtr<gfxQuartzSurface> targetSurface;
 
-  RefPtr<gfxContext> targetContext;
+  UniquePtrPtr<gfxContext> targetContext;
   if (gfxPlatform::GetPlatform()->SupportsAzureContentForType(gfx::BackendType::CAIRO)) {
     // This is dead code unless you mess with prefs, but keep it around for
     // debugging.
@@ -331,7 +330,6 @@ class nsAutoRetainUIKitObject {
           << "Window context problem 2 " << backingSize;
       return;
     }
-    dt->AddUserData(&gfxContext::sDontUseAsSourceKey, dt, nullptr);
     targetContext = gfxContext::CreateOrNull(dt);
   } else {
     MOZ_ASSERT_UNREACHABLE("COREGRAPHICS is the only supported backend");
@@ -348,15 +346,6 @@ class nsAutoRetainUIKitObject {
 
   // nsAutoRetainCocoaObject kungFuDeathGrip(self);
   bool painted = false;
-  if (mGeckoChild->GetLayerManager()->GetBackendType() == LayersBackend::LAYERS_BASIC) {
-    nsBaseWidget::AutoLayerManagerSetup setupLayerManager(mGeckoChild, targetContext,
-                                                          BufferMode::BUFFER_NONE);
-    painted = mGeckoChild->PaintWindow(region);
-  } else if (mGeckoChild->GetLayerManager()->GetBackendType() == LayersBackend::LAYERS_CLIENT) {
-    // We only need this so that we actually get DidPaintWindow fired
-    painted = mGeckoChild->PaintWindow(region);
-  }
-
   targetContext = nullptr;
   targetSurface = nullptr;
 
@@ -389,7 +378,8 @@ class nsAutoRetainUIKitObject {
 }
 @end
 
-nsWindow::nsWindow() : mNativeView(nullptr), mVisible(false), mParent(nullptr) {}
+nsWindow::nsWindow()
+    : mNativeView(nullptr), mVisible(false), mSizeMode(nsSizeMode_Normal), mParent(nullptr) {}
 
 nsWindow::~nsWindow() {
   [mNativeView widgetDestroyed];  // Safe if mNativeView is nil.
@@ -406,8 +396,8 @@ void nsWindow::TearDownView() {
 }
 
 bool nsWindow::IsTopLevel() {
-  return mWindowType == eWindowType_toplevel || mWindowType == eWindowType_dialog ||
-         mWindowType == eWindowType_invisible;
+  return mWindowType == WindowType::TopLevel || mWindowType == WindowType::Dialog ||
+         mWindowType == WindowType::Invisible;
 }
 
 //
@@ -415,7 +405,7 @@ bool nsWindow::IsTopLevel() {
 //
 
 nsresult nsWindow::Create(nsIWidget* aParent, nsNativeWidget aNativeParent,
-                          const LayoutDeviceIntRect& aRect, nsWidgetInitData* aInitData) {
+                          const LayoutDeviceIntRect& aRect, widget::InitData* aInitData) {
   ALOG("nsWindow[%p]::Create %p/%p [%d %d %d %d]", (void*)this, (void*)aParent,
        (void*)aNativeParent, aRect.x, aRect.y, aRect.width, aRect.height);
   nsWindow* parent = (nsWindow*)aParent;
@@ -443,8 +433,8 @@ nsresult nsWindow::Create(nsIWidget* aParent, nsNativeWidget aNativeParent,
        mBounds.height);
 
   // Set defaults which can be overriden from aInitData in BaseCreate
-  mWindowType = eWindowType_toplevel;
-  mBorderStyle = eBorderStyle_default;
+  mWindowType = WindowType::TopLevel;
+  mBorderStyle = BorderStyle::Default;
 
   Inherited::BaseCreate(aParent, aInitData);
 
@@ -492,16 +482,6 @@ void nsWindow::Destroy() {
   return NS_OK;
 }
 
-nsresult nsWindow::ConfigureChildren(const nsTArray<nsIWidget::Configuration>& config) {
-  for (uint32_t i = 0; i < config.Length(); ++i) {
-    nsWindow* childWin = (nsWindow*)config[i].mChild.get();
-    childWin->Resize(config[i].mBounds.x, config[i].mBounds.y, config[i].mBounds.width,
-                     config[i].mBounds.height, false);
-  }
-
-  return NS_OK;
-}
-
 void nsWindow::Show(bool aState) {
   if (aState != mVisible) {
     mNativeView.hidden = aState ? NO : YES;
@@ -583,7 +563,7 @@ void nsWindow::SetSizeMode(nsSizeMode aMode) {
 void nsWindow::Invalidate(const LayoutDeviceIntRect& aRect) {
   if (!mNativeView || !mVisible) return;
 
-  MOZ_RELEASE_ASSERT(GetLayerManager()->GetBackendType() != LayersBackend::LAYERS_CLIENT,
+  MOZ_RELEASE_ASSERT(GetLayerManager()->GetBackendType() != LayersBackend::LAYERS_WR,
                      "Shouldn't need to invalidate with accelerated OMTC layers!");
 
   [mNativeView setNeedsLayout];
@@ -692,7 +672,6 @@ void* nsWindow::GetNativeData(uint32_t aDataType) {
 
   switch (aDataType) {
     case NS_NATIVE_WIDGET:
-    case NS_NATIVE_DISPLAY:
       retVal = (void*)mNativeView;
       break;
 
@@ -712,10 +691,6 @@ void* nsWindow::GetNativeData(uint32_t aDataType) {
       retVal = 0;
       break;
 
-    case NS_NATIVE_PLUGIN_PORT:
-      // not implemented
-      break;
-
     case NS_RAW_NATIVE_IME_CONTEXT:
       retVal = GetPseudoIMEContext();
       if (retVal) {
