@@ -9,7 +9,7 @@
 #include "mozilla/RefPtr.h"
 #include "nscore.h"
 #include "nsString.h"
-#include "nsWindowBase.h"
+#include "nsWindow.h"
 #include "nsWindowDefs.h"
 #include "mozilla/Attributes.h"
 #include "mozilla/EventForwards.h"
@@ -39,7 +39,7 @@
 #define VK_OEM_102 0xE2
 #define VK_OEM_CLEAR 0xFE
 
-class nsIIdleServiceInternal;
+class nsIUserIdleServiceInternal;
 
 namespace mozilla {
 namespace widget {
@@ -124,7 +124,7 @@ class MOZ_STACK_CLASS UniCharsAndModifiers final {
   nsAutoString mChars;
   // 5 is enough number for normal keyboard layout handling.  On Windows,
   // a dead key sequence may cause inputting up to 5 characters per key press.
-  AutoTArray<Modifiers, 5> mModifiers;
+  CopyableAutoTArray<Modifiers, 5> mModifiers;
 };
 
 struct DeadKeyEntry {
@@ -382,10 +382,10 @@ class MOZ_STACK_CLASS NativeKey final {
     MSG GetCharMsg(HWND aWnd) const {
       MSG msg;
       msg.hwnd = aWnd;
-      msg.message =
-          mIsDeadKey && mIsSysKey
-              ? WM_SYSDEADCHAR
-              : mIsDeadKey ? WM_DEADCHAR : mIsSysKey ? WM_SYSCHAR : WM_CHAR;
+      msg.message = mIsDeadKey && mIsSysKey ? WM_SYSDEADCHAR
+                    : mIsDeadKey            ? WM_DEADCHAR
+                    : mIsSysKey             ? WM_SYSCHAR
+                                            : WM_CHAR;
       msg.wParam = static_cast<WPARAM>(mCharCode);
       msg.lParam = static_cast<LPARAM>(mScanCode << 16);
       msg.time = 0;
@@ -394,7 +394,7 @@ class MOZ_STACK_CLASS NativeKey final {
     }
   };
 
-  NativeKey(nsWindowBase* aWidget, const MSG& aMessage,
+  NativeKey(nsWindow* aWidget, const MSG& aMessage,
             const ModifierKeyState& aModKeyState,
             HKL aOverrideKeyboardLayout = 0,
             nsTArray<FakeCharMsg>* aFakeCharMsgs = nullptr);
@@ -441,6 +441,7 @@ class MOZ_STACK_CLASS NativeKey final {
    */
   static bool IsControlChar(char16_t aChar);
 
+  bool IsShift() const { return mModKeyState.IsShift(); }
   bool IsControl() const { return mModKeyState.IsControl(); }
   bool IsAlt() const { return mModKeyState.IsAlt(); }
   bool MaybeEmulatingAltGraph() const;
@@ -473,7 +474,7 @@ class MOZ_STACK_CLASS NativeKey final {
   // mReceivedMsg is set when another instance starts to handle the message
   // unexpectedly.
   MSG mReceivedMsg;
-  RefPtr<nsWindowBase> mWidget;
+  RefPtr<nsWindow> mWidget;
   RefPtr<TextEventDispatcher> mDispatcher;
   HKL mKeyboardLayout;
   MSG mMsg;
@@ -604,13 +605,30 @@ class MOZ_STACK_CLASS NativeKey final {
    */
   bool IsIMEDoingKakuteiUndo() const;
 
+  /**
+   * This returns true if user types a number key in numpad with Alt key
+   * to input a Unicode character from its scalar value.
+   * Note that inputting Unicode scalar value is available without NumLock.
+   * Therefore, this returns true even if user presses a function key on
+   * numpad without NumLock, but that may be intended to perform a shortcut
+   * key like Alt + Home.
+   */
+  bool MaybeTypingUnicodeScalarValue() const {
+    return !mIsExtended && IsSysKeyDownOrKeyUpMessage() && IsAlt() &&
+           !IsControl() && !IsShift() &&
+           ((mScanCode >= 0x004F && mScanCode <= 0x0052) ||  // Numpad0-3
+            (mScanCode >= 0x004B && mScanCode <= 0x004D) ||  // Numpad4-6
+            (mScanCode >= 0x0047 && mScanCode <= 0x0049));   // Numpad7-9
+  }
+
   bool IsKeyDownMessage() const {
-    return (mMsg.message == WM_KEYDOWN || mMsg.message == WM_SYSKEYDOWN ||
-            mMsg.message == MOZ_WM_KEYDOWN);
+    return mMsg.message == WM_KEYDOWN || mMsg.message == WM_SYSKEYDOWN;
   }
   bool IsKeyUpMessage() const {
-    return (mMsg.message == WM_KEYUP || mMsg.message == WM_SYSKEYUP ||
-            mMsg.message == MOZ_WM_KEYUP);
+    return mMsg.message == WM_KEYUP || mMsg.message == WM_SYSKEYUP;
+  }
+  bool IsSysKeyDownOrKeyUpMessage() const {
+    return mMsg.message == WM_SYSKEYDOWN || mMsg.message == WM_SYSKEYUP;
   }
   bool IsCharOrSysCharMessage(const MSG& aMSG) const {
     return IsCharOrSysCharMessage(aMSG.message);
@@ -642,9 +660,6 @@ class MOZ_STACK_CLASS NativeKey final {
   bool IsFollowedByPrintableCharMessage() const;
   bool IsFollowedByPrintableCharOrSysCharMessage() const;
   bool IsFollowedByDeadCharMessage() const;
-  bool IsKeyMessageOnPlugin() const {
-    return (mMsg.message == MOZ_WM_KEYDOWN || mMsg.message == MOZ_WM_KEYUP);
-  }
   bool IsPrintableCharMessage(const MSG& aMSG) const {
     return aMSG.message == WM_CHAR &&
            !IsControlChar(static_cast<char16_t>(aMSG.wParam));
@@ -702,17 +717,8 @@ class MOZ_STACK_CLASS NativeKey final {
    * Initializes the aKeyEvent with the information stored in the instance.
    */
   nsEventStatus InitKeyEvent(WidgetKeyboardEvent& aKeyEvent,
-                             const ModifierKeyState& aModKeyState,
-                             const MSG* aMsgSentToPlugin = nullptr) const;
-  nsEventStatus InitKeyEvent(WidgetKeyboardEvent& aKeyEvent,
-                             const MSG* aMsgSentToPlugin = nullptr) const;
-
-  /**
-   * MaybeInitPluginEventOfKeyEvent() may initialize aKeyEvent::mPluginEvent
-   * with aMsgSentToPlugin if it's necessary.
-   */
-  void MaybeInitPluginEventOfKeyEvent(WidgetKeyboardEvent& aKeyEvent,
-                                      const MSG& aMsgSentToPlugin) const;
+                             const ModifierKeyState& aModKeyState) const;
+  nsEventStatus InitKeyEvent(WidgetKeyboardEvent& aKeyEvent) const;
 
   /**
    * Dispatches a command event for aEventCommand.
@@ -733,14 +739,6 @@ class MOZ_STACK_CLASS NativeKey final {
    */
   bool DispatchKeyPressEventsWithoutCharMessage() const;
 
-  /**
-   * MaybeDispatchPluginEventsForRemovedCharMessages() dispatches plugin events
-   * for removed char messages when a windowless plugin has focus.
-   * Returns true if the widget is destroyed or blurred during dispatching a
-   * plugin event.
-   */
-  bool MaybeDispatchPluginEventsForRemovedCharMessages() const;
-
   /**
    * Checkes whether the key event down message is handled without following
    * WM_CHAR messages.  For example, if following WM_CHAR message indicates
@@ -923,7 +921,7 @@ class KeyboardLayout {
   /**
    * Implementation of nsIWidget::SynthesizeNativeKeyEvent().
    */
-  nsresult SynthesizeNativeKeyEvent(nsWindowBase* aWidget,
+  nsresult SynthesizeNativeKeyEvent(nsWindow* aWidget,
                                     int32_t aNativeKeyboardLayout,
                                     int32_t aNativeKeyCode,
                                     uint32_t aModifierFlags,
@@ -935,7 +933,7 @@ class KeyboardLayout {
   ~KeyboardLayout();
 
   static KeyboardLayout* sInstance;
-  static nsIIdleServiceInternal* sIdleService;
+  static nsIUserIdleServiceInternal* sIdleService;
 
   struct DeadKeyTableListEntry {
     DeadKeyTableListEntry* next;
@@ -1060,7 +1058,7 @@ class RedirectedKeyDownMessageManager {
    */
   class MOZ_STACK_CLASS AutoFlusher final {
    public:
-    AutoFlusher(nsWindowBase* aWidget, const MSG& aMsg)
+    AutoFlusher(nsWindow* aWidget, const MSG& aMsg)
         : mCancel(!RedirectedKeyDownMessageManager::IsRedirectedMessage(aMsg)),
           mWidget(aWidget),
           mMsg(aMsg) {}
@@ -1081,7 +1079,7 @@ class RedirectedKeyDownMessageManager {
 
    private:
     bool mCancel;
-    RefPtr<nsWindowBase> mWidget;
+    RefPtr<nsWindow> mWidget;
     const MSG& mMsg;
   };
 
