@@ -6,21 +6,31 @@
 #ifndef ToastNotification_h__
 #define ToastNotification_h__
 
+#include "mozilla/Maybe.h"
+#include "mozilla/MozPromise.h"
 #include "nsIAlertsService.h"
 #include "nsIObserver.h"
 #include "nsIThread.h"
+#include "nsIWindowsAlertsService.h"
 #include "nsRefPtrHashtable.h"
 
 namespace mozilla {
 namespace widget {
 
+struct ToastHandledResolve {
+  const nsString launchUrl;
+  const nsString privilegedName;
+};
+using ToastHandledPromise = MozPromise<ToastHandledResolve, bool, true>;
+
 class ToastNotificationHandler;
 
-class ToastNotification final : public nsIAlertsService,
+class ToastNotification final : public nsIWindowsAlertsService,
                                 public nsIAlertsDoNotDisturb,
                                 public nsIObserver {
  public:
   NS_DECL_NSIALERTSSERVICE
+  NS_DECL_NSIWINDOWSALERTSSERVICE
   NS_DECL_NSIALERTSDONOTDISTURB
   NS_DECL_NSIOBSERVER
   NS_DECL_ISUPPORTS
@@ -38,9 +48,20 @@ class ToastNotification final : public nsIAlertsService,
 
  protected:
   virtual ~ToastNotification();
+  bool EnsureAumidRegistered();
+
+  static bool AssignIfMsixAumid(Maybe<nsAutoString>& aAumid);
+  static bool AssignIfNsisAumid(nsAutoString& aInstallHash,
+                                Maybe<nsAutoString>& aAumid);
+  static bool RegisterRuntimeAumid(nsAutoString& aInstallHash,
+                                   Maybe<nsAutoString>& aAumid);
+
+  RefPtr<ToastHandledPromise> VerifyTagPresentOrFallback(
+      const nsAString& aWindowsTag);
+  static void SignalComNotificationHandled(const nsAString& aWindowsTag);
 
   nsRefPtrHashtable<nsStringHashKey, ToastNotificationHandler> mActiveHandlers;
-  nsCOMPtr<nsIThread> mBackgroundThread;
+  Maybe<nsAutoString> mAumid;
   bool mSuppressForScreenSharing = false;
 };
 
